var powerbi;
(function (powerbi) {
    var visuals;
    (function (visuals) {
        var BrushChart1447292826280;
        (function (BrushChart1447292826280) {
            // Visual definition
            var BrushChart = (function () {
                function BrushChart() {
                    // default colors
                    this.brushFillColors = { detailsFillColour: '#005496', slicerFillColour: '#BBBDC0' };
                }
                // Convert dataview object to model
                BrushChart.converter = function (dataView) {
                    var catDv = dataView.categorical;
                    if (catDv.categories && catDv.values) {
                        var cat = catDv.categories[0];
                        var catValues = cat.values;
                        var values = catDv.values;
                        var dataPoints = [];
                        for (var i = 0, len = catValues.length; i < len; i++) {
                            if ((catValues[i] instanceof Date) && !isNaN(values[0].values[i])) {
                                dataPoints.push({
                                    x: catValues[i],
                                    y: values[0].values[i]
                                });
                            }
                        }
                        var viewModel = {
                            points: dataPoints,
                            xLabel: cat.source.displayName,
                            yLabel: values[0].source.displayName,
                            yFormat: values[0].source.format
                        };
                        return viewModel;
                    }
                };
                // Initialize visual components
                BrushChart.prototype.init = function (options) {
                    var element = options.element;
                    this.svg = d3.select(element.get(0)).append('svg');
                    this.rect = this.svg.append("defs").append("clipPath").attr("id", "clip").append("rect");
                    this.focus = this.svg.append('g');
                    this.context = this.svg.append('g');
                    this.focusArea = this.focus.append("path");
                    this.contextArea = this.context.append("path");
                    this.focusX = this.focus.append('g');
                    this.contextX = this.context.append('g');
                    this.focusY = this.focus.append('g');
                    this.axisYRect = this.focusY.append('rect');
                    this.axisYRectRight = this.focusY.append('rect');
                    this.contextY = this.context.append('g');
                    this.host = options.host;
                };
                // validate the category and y axis data types
                BrushChart.prototype.validateAxis = function (dataView, host, viewport) {
                    var warnings = [];
                    // category field is not datetime
                    if (dataView.categorical.categories) {
                        if (!dataView.categorical.categories[0].source.type.dateTime) {
                            warnings.push({
                                code: 'NotDateCategoryField',
                                getMessages: function (resourceProvider) {
                                    var visualMessage = {
                                        message: BrushChart.category_error_message,
                                        title: BrushChart.category_error_title,
                                        detail: BrushChart.category_error_message,
                                    };
                                    return visualMessage;
                                }
                            });
                            this.host.setWarnings(warnings);
                            this.generateError(1, viewport);
                        }
                    }
                    if (dataView.categorical.values) {
                        // Y is not numeric
                        if (!dataView.categorical.values[0].source.type.numeric) {
                            warnings.push({
                                code: 'NotNumericYAxisField',
                                getMessages: function (resourceProvider) {
                                    var visualMessage = {
                                        message: BrushChart.yaxis_error_message,
                                        title: BrushChart.yaxis_error_title,
                                        detail: BrushChart.yaxis_error_message,
                                    };
                                    return visualMessage;
                                }
                            });
                            this.host.setWarnings(warnings);
                            this.generateError(2, viewport);
                        }
                        else {
                            this.cleanError();
                        }
                    }
                };
                // add error message
                BrushChart.prototype.generateError = function (type, viewport) {
                    var erroMessage;
                    switch (type) {
                        case 1:
                            erroMessage = BrushChart.category_error_message;
                            break;
                        case 2:
                            erroMessage = BrushChart.category_error_title;
                            break;
                        default:
                            erroMessage = "An error occured. Please contact the support.";
                            break;
                    }
                    this.cleanError();
                    this.svg.append('rect').attr('class', 'errorRectangle').attr('width', viewport.width).attr('height', viewport.height).attr('fill', 'white');
                    var text = this.svg.append('text').attr('class', 'errorMessage').attr('text-anchor', 'middle').style("font", "16px sans-serif").attr('transform', 'translate(' + viewport.width / 2 + ',' + viewport.height / 2 + ')').attr('fill', 'red');
                    text.text(erroMessage);
                };
                BrushChart.prototype.cleanError = function () {
                    this.svg.selectAll(".errorMessage").remove();
                    this.svg.selectAll(".errorRectangle").remove();
                };
                // Update visual components
                BrushChart.prototype.update = function (options) {
                    if (!options.dataViews || !options.dataViews[0])
                        return;
                    this.dataView = options.dataViews[0];
                    var viewport = options.viewport;
                    // margins settings
                    var margin = { top: 10, right: 10, bottom: 110, left: 40 }, margin2 = { top: viewport.height - 85, right: 10, bottom: 20, left: 40 }, width = viewport.width - margin.left - margin.right, height = viewport.height - margin.top - margin.bottom, height2 = viewport.height - margin2.top - margin2.bottom;
                    this.svg.attr("width", width + margin.left + margin.right).attr("height", height + margin.top + margin.bottom).style("position", "absolute").style("font", "10px sans-serif");
                    var viewModel;
                    // validate category and y axis
                    this.validateAxis(this.dataView, this.host, viewport);
                    viewModel = BrushChart.converter(this.dataView);
                    if (viewModel) {
                        var data = viewModel.points;
                        // apply visual style and set functionalities
                        var x = d3.time.scale().range([0, width]), x2 = d3.time.scale().range([0, width]), y = d3.scale.linear().range([height, 0]), y2 = d3.scale.linear().range([height2, 0]);
                        var xAxis = d3.svg.axis().scale(x).orient("bottom"), xAxis2 = d3.svg.axis().scale(x2).orient("bottom"), yAxis = d3.svg.axis().scale(y).orient("left");
                        var area = d3.svg.area().interpolate("monotone").x(function (d) {
                            return x(d.x);
                        }).y0(height).y1(function (d) {
                            return y(+d.y);
                        });
                        var area2 = d3.svg.area().interpolate("monotone").x(function (d) {
                            return x2(d.x);
                        }).y0(height2).y1(function (d) {
                            return y2(+d.y);
                        });
                        var generateTooltipInfo = function (extentX, viewModel) {
                            var ySum = 0;
                            var data = viewModel.points;
                            for (var i = 0; i < data.length; i++) {
                                if (extentX[0] <= data[i].x && data[i].x <= extentX[1]) {
                                    ySum = ySum + data[i].y;
                                }
                            }
                            return [
                                {
                                    displayName: 'Time Range (' + viewModel.xLabel + ')',
                                    value: extentX[0].toDateString() + ' - ' + extentX[1].toDateString()
                                },
                                {
                                    displayName: 'Total (' + viewModel.yLabel + ')',
                                    value: visuals.valueFormatter.format(ySum, viewModel.yFormat)
                                }
                            ];
                        };
                        var brush = d3.svg.brush().x(x2).on("brush", function brushed() {
                            x.domain(brush.empty() ? x2.domain() : brush.extent());
                            focus.select(".area").empty();
                            focus.select(".area").attr("d", area);
                            focus.select(".x.axis").call(xAxis);
                            // generate back the tooltip
                            var tooltip = generateTooltipInfo(brush.extent(), viewModel);
                            visuals.TooltipManager.addTooltip(focus, function (tooltipEvent) { return tooltip; });
                        }, false);
                        this.rect.attr("width", width).attr("height", height);
                        this.axisYRect.attr("width", margin.left).attr("height", height).attr("x", -1 * margin.left).attr("fill", "#fff");
                        this.axisYRectRight.attr("width", margin.left).attr("height", height).attr("x", width).attr("fill", "#fff");
                        var focus = this.focus;
                        focus.attr("class", "focus").attr("transform", "translate(" + margin.left + "," + margin.top + ")");
                        var context = this.context;
                        context.attr("class", "context").attr("transform", "translate(" + margin2.left + "," + margin2.top + ")");
                        x.domain(d3.extent(data.map(function (d) {
                            return d.x;
                        })));
                        y.domain([0, d3.max(data.map(function (d) {
                            return +d.y;
                        }))]);
                        x2.domain(x.domain());
                        y2.domain(y.domain());
                        this.focusArea.datum(data).attr("class", "area").attr("d", area).attr('fill', this.getFill(this.dataView, 'fill').solid.color).attr('fill-opacity', .5).attr('clip-path', 'url(#clip)').attr('fill-opacity', .4).attr('stroke', this.getFill(this.dataView, 'fill').solid.color).attr('stroke-width', 1.8);
                        this.focusX.attr("class", "x axis").attr("transform", "translate(0," + height + ")").call(xAxis);
                        this.focusY.attr("class", "y axis").call(yAxis);
                        this.focusX.select('path').attr('fill', 'none').attr('stroke', '#000').attr('shape-rendering', 'crispEdges');
                        this.focusX.select('line').attr('fill', 'none').attr('stroke', '#000').attr('shape-rendering', 'crispEdges');
                        this.focusY.select('path').attr('fill', 'none').attr('stroke', '#000').attr('shape-rendering', 'crispEdges');
                        this.focusY.select('line').attr('fill', 'none').attr('stroke', '#000').attr('shape-rendering', 'crispEdges');
                        this.focusY.attr('background-color', '#fff');
                        this.focusY.append("rect");
                        this.contextArea.datum(data).attr("class", "area brush").attr("d", area2).attr('fill', this.getFill(this.dataView, 'fill1').solid.color).attr('clip-path', 'url(#clip)').attr('fill-opacity', .4).attr('stroke', this.getFill(this.dataView, 'fill1').solid.color).attr('stroke-width', 1.8);
                        this.contextX.attr("class", "x axis").attr("transform", "translate(0," + height2 + ")").call(xAxis2);
                        this.contextY.attr("class", "x brush").call(brush).selectAll('rect').attr('y', -6).attr('height', height2 + 7).attr('drag-resize-disabled', true);
                        this.contextX.select('path').attr('fill', 'none').attr('stroke', '#000').attr('shape-rendering', 'crispEdges');
                        this.contextX.select('line').attr('fill', 'none').attr('stroke', '#000').attr('shape-rendering', 'crispEdges');
                        this.contextY.select('.extent').attr('stroke', '#fff').attr('fill-opacity', '.125').attr('shape-rendering', 'crispEdges');
                        visuals.TooltipManager.addTooltip(this.focusArea, function (tooltipEvent) { return generateTooltipInfo(x2.domain(), viewModel); });
                    }
                };
                // Define visual properties
                BrushChart.prototype.enumerateObjectInstances = function (options) {
                    var instances = [];
                    switch (options.objectName) {
                        case 'label':
                            var label = {
                                objectName: 'label',
                                displayName: 'BrushChart Colors',
                                selector: null,
                                properties: {
                                    fill: this.getFill(this.dataView, 'fill'),
                                    fill1: this.getFill(this.dataView, 'fill1')
                                }
                            };
                            instances.push(label);
                            break;
                    }
                    return instances;
                };
                // Get properties values
                BrushChart.prototype.getFill = function (dataView, fieldName) {
                    if (dataView && dataView.metadata.objects) {
                        var label = dataView.metadata.objects['label'];
                        if (label) {
                            if (label[fieldName])
                                return label[fieldName];
                        }
                    }
                    return { solid: { color: fieldName === 'fill' ? this.brushFillColors.detailsFillColour : this.brushFillColors.slicerFillColour } };
                };
                // Brush chart capabilities
                BrushChart.capabilities = {
                    // roles
                    dataRoles: [
                        {
                            name: 'Category',
                            kind: powerbi.VisualDataRoleKind.Grouping,
                        },
                        {
                            name: 'Y',
                            kind: powerbi.VisualDataRoleKind.Measure,
                        },
                    ],
                    // mapping
                    dataViewMappings: [{
                        conditions: [
                            { 'Category': { max: 1 }, 'Y': { max: 0 } },
                            { 'Category': { max: 1 }, 'Y': { min: 1, max: 1 } }
                        ],
                        categorical: {
                            categories: { for: { in: 'Category' } },
                            values: {
                                select: [{ bind: { to: 'Y' } }]
                            }
                        }
                    }],
                    // Visual properties
                    objects: {
                        general: {
                            displayName: powerbi.data.createDisplayNameGetter('Visual_General'),
                            properties: {
                                formatString: {
                                    type: { formatting: { formatString: true } },
                                },
                            },
                        },
                        label: {
                            displayName: 'BrushChart Colors',
                            properties: {
                                fill: {
                                    displayName: 'Details View',
                                    type: { fill: { solid: { color: true } } }
                                },
                                fill1: {
                                    displayName: 'Time Slicer',
                                    type: { fill: { solid: { color: true } } }
                                }
                            }
                        }
                    },
                    sorting: {
                        custom: {}
                    }
                };
                BrushChart.category_error_message = 'The category data should be a date.';
                BrushChart.yaxis_error_message = 'The Y axis should be numeric.';
                BrushChart.category_error_title = 'Invalid Category';
                BrushChart.yaxis_error_title = 'Invalid Y Axis';
                return BrushChart;
            })();
            BrushChart1447292826280.BrushChart = BrushChart;
        })(BrushChart1447292826280 = visuals.BrushChart1447292826280 || (visuals.BrushChart1447292826280 = {}));
    })(visuals = powerbi.visuals || (powerbi.visuals = {}));
})(powerbi || (powerbi = {}));
var powerbi;
(function (powerbi) {
    var visuals;
    (function (visuals) {
        var plugins;
        (function (plugins) {
            plugins.BrushChart1447292826280 = {
                name: 'BrushChart1447292826280',
                class: 'BrushChart1447292826280',
                capabilities: powerbi.visuals.BrushChart1447292826280.BrushChart.capabilities,
                custom: true,
                create: function () { return new powerbi.visuals.BrushChart1447292826280.BrushChart(); }
            };
        })(plugins = visuals.plugins || (visuals.plugins = {}));
    })(visuals = powerbi.visuals || (powerbi.visuals = {}));
})(powerbi || (powerbi = {}));
