/*
Power BI Visualizations

Copyright (c) Microsoft Corporation

All rights reserved. 

MIT License

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
*/
//// <reference path="../../_references.ts"/>
var powerbi;
(function (powerbi) {
    var visuals;
    (function (visuals) {
        var ChicletSlicer1448559807354;
        (function (ChicletSlicer1448559807354) {
            var createClassAndSelector = jsCommon.CssConstants.createClassAndSelector;
            var PixelConverter = jsCommon.PixelConverter;
            var TableViewFactory;
            (function (TableViewFactory) {
                function createTableView(options) {
                    return new TableView(options);
                }
                TableViewFactory.createTableView = createTableView;
            })(TableViewFactory = ChicletSlicer1448559807354.TableViewFactory || (ChicletSlicer1448559807354.TableViewFactory = {}));
            var TableView = (function () {
                function TableView(options) {
                    this.options = $.extend(true, {}, options);
                    this.options.baseContainer
                        .style('overflow-y', 'auto')
                        .attr('drag-resize-disabled', true);
                    this.scrollContainer = options.baseContainer
                        .append('div')
                        .attr('class', 'scrollRegion');
                    this.visibleGroupContainer = this.scrollContainer
                        .append('div')
                        .attr('class', 'visibleGroup');
                    TableView.SetDefaultOptions(options);
                }
                TableView.SetDefaultOptions = function (options) {
                    options.rowHeight = options.rowHeight || TableView.defaultRowHeight;
                };
                TableView.prototype.rowHeight = function (rowHeight) {
                    this.options.rowHeight = Math.ceil(rowHeight);
                    return this;
                };
                TableView.prototype.columnWidth = function (columnWidth) {
                    this.options.columnWidth = Math.ceil(columnWidth);
                    return this;
                };
                TableView.prototype.orientation = function (orientation) {
                    this.options.orientation = orientation;
                    return this;
                };
                TableView.prototype.rows = function (rows) {
                    this.options.rows = Math.ceil(rows);
                    return this;
                };
                TableView.prototype.columns = function (columns) {
                    this.options.columns = Math.ceil(columns);
                    return this;
                };
                TableView.prototype.data = function (data, getDatumIndex, dataReset) {
                    if (dataReset === void 0) { dataReset = false; }
                    this._data = data;
                    this.getDatumIndex = getDatumIndex;
                    this.setTotalRows();
                    if (dataReset) {
                        $(this.options.baseContainer.node()).scrollTop(0);
                    }
                    return this;
                };
                TableView.prototype.viewport = function (viewport) {
                    this.options.viewport = viewport;
                    return this;
                };
                TableView.prototype.empty = function () {
                    this._data = [];
                    this.render();
                };
                TableView.prototype.setTotalRows = function () {
                    var count = this._data.length;
                    var rows = Math.min(this.options.rows, count);
                    var columns = Math.min(this.options.columns, count);
                    if ((columns > 0) && (rows > 0)) {
                        this._totalColumns = columns;
                        this._totalRows = rows;
                    }
                    else if (rows > 0) {
                        this._totalRows = rows;
                        this._totalColumns = Math.ceil(count / rows);
                    }
                    else if (columns > 0) {
                        this._totalColumns = columns;
                        this._totalRows = Math.ceil(count / columns);
                    }
                    else {
                        this._totalColumns = TableView.defaultColumns;
                        this._totalRows = Math.ceil(count / TableView.defaultColumns);
                    }
                    if (this.options.orientation === Orientation.HORIZONTAL) {
                        this._totalRows = 1;
                        this._totalColumns = count;
                    }
                };
                TableView.prototype.render = function () {
                    var options = this.options;
                    var visibleGroupContainer = this.visibleGroupContainer;
                    var rowHeight = options.rowHeight || TableView.defaultRowHeight;
                    var groupedData = [];
                    for (var i = 0; i < this._totalRows; i++) {
                        var k = i * this._totalColumns;
                        groupedData.push(this._data.slice(k, k + this._totalColumns));
                    }
                    visibleGroupContainer.selectAll(".row").remove();
                    var cellSelection = visibleGroupContainer.selectAll(".row")
                        .data(groupedData)
                        .enter()
                        .append("div")
                        .classed('row', true)
                        .selectAll(".cell")
                        .data(function (d) { return d; });
                    cellSelection
                        .enter()
                        .append('div')
                        .classed('cell', true)
                        .call(function (d) { return options.enter(d); });
                    cellSelection.order();
                    var cellUpdateSelection = visibleGroupContainer.selectAll('.cell:not(.transitioning)');
                    cellUpdateSelection.call(function (d) { return options.update(d); });
                    cellUpdateSelection.style({ 'height': (rowHeight > 0) ? rowHeight + 'px' : 'auto' });
                    cellUpdateSelection.style({
                        'width': (options.columnWidth > 0) ? options.columnWidth + 'px' : (100 / this._totalColumns) + '%'
                    });
                    cellSelection
                        .exit()
                        .call(function (d) { return options.exit(d); })
                        .remove();
                };
                TableView.defaultRowHeight = 0;
                TableView.defaultColumns = 1;
                return TableView;
            })();
            ChicletSlicer1448559807354.chicletSlicerProps = {
                general: {
                    orientation: { objectName: 'general', propertyName: 'orientation' },
                    columns: { objectName: 'general', propertyName: 'columns' },
                    rows: { objectName: 'general', propertyName: 'rows' },
                    showDisabled: { objectName: 'general', propertyName: 'showDisabled' },
                    multiselect: { objectName: 'general', propertyName: 'multiselect' },
                },
                header: {
                    show: { objectName: 'header', propertyName: 'show' },
                    title: { objectName: 'header', propertyName: 'title' },
                    fontColor: { objectName: 'header', propertyName: 'fontColor' },
                    background: { objectName: 'header', propertyName: 'background' },
                    outline: { objectName: 'header', propertyName: 'outline' },
                    textSize: { objectName: 'header', propertyName: 'textSize' },
                    outlineColor: { objectName: 'header', propertyName: 'outlineColor' },
                    outlineWeight: { objectName: 'header', propertyName: 'outlineWeight' }
                },
                rows: {
                    fontColor: { objectName: 'rows', propertyName: 'fontColor' },
                    textSize: { objectName: 'rows', propertyName: 'textSize' },
                    height: { objectName: 'rows', propertyName: 'height' },
                    width: { objectName: 'rows', propertyName: 'width' },
                    background: { objectName: 'rows', propertyName: 'background' },
                    transparency: { objectName: 'rows', propertyName: 'transparency' },
                    selectedColor: { objectName: 'rows', propertyName: 'selectedColor' },
                    unselectedColor: { objectName: 'rows', propertyName: 'unselectedColor' },
                    disabledColor: { objectName: 'rows', propertyName: 'disabledColor' },
                    outline: { objectName: 'rows', propertyName: 'outline' },
                    outlineColor: { objectName: 'rows', propertyName: 'outlineColor' },
                    outlineWeight: { objectName: 'rows', propertyName: 'outlineWeight' },
                    borderStyle: { objectName: 'rows', propertyName: 'borderStyle' },
                },
                images: {
                    imageSplit: { objectName: 'images', propertyName: 'imageSplit' },
                    stretchImage: { objectName: 'images', propertyName: 'stretchImage' },
                    bottomImage: { objectName: 'images', propertyName: 'bottomImage' },
                },
                selectedPropertyIdentifier: { objectName: 'general', propertyName: 'selected' },
                filterPropertyIdentifier: { objectName: 'general', propertyName: 'filter' },
                formatString: { objectName: 'general', propertyName: 'formatString' },
            };
            var ChicletBorderStyle;
            (function (ChicletBorderStyle) {
                ChicletBorderStyle.ROUNDED = 'Rounded';
                ChicletBorderStyle.CUT = 'Cut';
                ChicletBorderStyle.SQUARE = 'Square';
                ChicletBorderStyle.type = powerbi.createEnumType([
                    { value: ChicletBorderStyle.ROUNDED, displayName: ChicletBorderStyle.ROUNDED },
                    { value: ChicletBorderStyle.CUT, displayName: ChicletBorderStyle.CUT },
                    { value: ChicletBorderStyle.SQUARE, displayName: ChicletBorderStyle.SQUARE },
                ]);
            })(ChicletBorderStyle || (ChicletBorderStyle = {}));
            var ChicletSlicerShowDisabled;
            (function (ChicletSlicerShowDisabled) {
                ChicletSlicerShowDisabled.INPLACE = 'Inplace';
                ChicletSlicerShowDisabled.BOTTOM = 'Bottom';
                ChicletSlicerShowDisabled.HIDE = 'Hide';
                ChicletSlicerShowDisabled.type = powerbi.createEnumType([
                    { value: ChicletSlicerShowDisabled.INPLACE, displayName: ChicletSlicerShowDisabled.INPLACE },
                    { value: ChicletSlicerShowDisabled.BOTTOM, displayName: ChicletSlicerShowDisabled.BOTTOM },
                    { value: ChicletSlicerShowDisabled.HIDE, displayName: ChicletSlicerShowDisabled.HIDE },
                ]);
            })(ChicletSlicerShowDisabled || (ChicletSlicerShowDisabled = {}));
            var Orientation;
            (function (Orientation) {
                Orientation.HORIZONTAL = 'Horizontal';
                Orientation.VERTICAL = 'Vertical';
                Orientation.type = powerbi.createEnumType([
                    { value: Orientation.HORIZONTAL, displayName: Orientation.HORIZONTAL },
                    { value: Orientation.VERTICAL, displayName: Orientation.VERTICAL }
                ]);
            })(Orientation || (Orientation = {}));
            var ChicletSlicer = (function () {
                function ChicletSlicer(options) {
                    if (options) {
                        if (options.behavior) {
                            this.behavior = options.behavior;
                        }
                    }
                    if (!this.behavior) {
                        this.behavior = new ChicletSlicerWebBehavior();
                    }
                }
                ChicletSlicer.DefaultStyleProperties = function () {
                    return {
                        general: {
                            orientation: Orientation.VERTICAL,
                            columns: 3,
                            rows: 0,
                            multiselect: true,
                            showDisabled: ChicletSlicerShowDisabled.INPLACE
                        },
                        margin: {
                            top: 50,
                            bottom: 50,
                            right: 50,
                            left: 50
                        },
                        header: {
                            borderBottomWidth: 1,
                            show: true,
                            outline: 'BottomOnly',
                            fontColor: '#a6a6a6',
                            background: '#ffffff',
                            textSize: 10,
                            outlineColor: '#a6a6a6',
                            outlineWeight: 1,
                            title: '',
                        },
                        headerText: {
                            marginLeft: 8,
                            marginTop: 0
                        },
                        slicerText: {
                            textSize: 10,
                            height: 0,
                            width: 0,
                            fontColor: '#666666',
                            hoverColor: '#212121',
                            selectedColor: '#BDD7EE',
                            unselectedColor: '#ffffff',
                            disabledColor: 'grey',
                            marginLeft: 8,
                            outline: 'Frame',
                            background: '#ffffff',
                            transparency: 0,
                            outlineColor: '#000000',
                            outlineWeight: 1,
                            borderStyle: 'Cut',
                        },
                        slicerItemContainer: {
                            marginTop: 5,
                            marginLeft: 0,
                        },
                        images: {
                            imageSplit: 50,
                            stretchImage: false,
                            bottomImage: false
                        }
                    };
                };
                ChicletSlicer.converter = function (dataView, localizedSelectAllText, interactivityService) {
                    if (!dataView ||
                        !dataView.categorical ||
                        !dataView.categorical.categories ||
                        !dataView.categorical.categories[0] ||
                        !dataView.categorical.categories[0].values ||
                        !(dataView.categorical.categories[0].values.length > 0)) {
                        return;
                    }
                    var converter = new ChicletSlicerChartConversion.ChicletSlicerConverter(dataView, interactivityService);
                    converter.convert();
                    var slicerData;
                    var defaultSettings = this.DefaultStyleProperties();
                    var objects = dataView.metadata.objects;
                    if (objects) {
                        defaultSettings.general.orientation = powerbi.DataViewObjects.getValue(objects, ChicletSlicer1448559807354.chicletSlicerProps.general.orientation, defaultSettings.general.orientation);
                        defaultSettings.general.columns = powerbi.DataViewObjects.getValue(objects, ChicletSlicer1448559807354.chicletSlicerProps.general.columns, defaultSettings.general.columns);
                        defaultSettings.general.rows = powerbi.DataViewObjects.getValue(objects, ChicletSlicer1448559807354.chicletSlicerProps.general.rows, defaultSettings.general.rows);
                        defaultSettings.general.multiselect = powerbi.DataViewObjects.getValue(objects, ChicletSlicer1448559807354.chicletSlicerProps.general.multiselect, defaultSettings.general.multiselect);
                        defaultSettings.general.showDisabled = powerbi.DataViewObjects.getValue(objects, ChicletSlicer1448559807354.chicletSlicerProps.general.showDisabled, defaultSettings.general.showDisabled);
                        defaultSettings.header.show = powerbi.DataViewObjects.getValue(objects, ChicletSlicer1448559807354.chicletSlicerProps.header.show, defaultSettings.header.show);
                        defaultSettings.header.title = powerbi.DataViewObjects.getValue(objects, ChicletSlicer1448559807354.chicletSlicerProps.header.title, defaultSettings.header.title);
                        defaultSettings.header.fontColor = powerbi.DataViewObjects.getFillColor(objects, ChicletSlicer1448559807354.chicletSlicerProps.header.fontColor, defaultSettings.header.fontColor);
                        defaultSettings.header.background = powerbi.DataViewObjects.getFillColor(objects, ChicletSlicer1448559807354.chicletSlicerProps.header.background, defaultSettings.header.background);
                        defaultSettings.header.textSize = powerbi.DataViewObjects.getValue(objects, ChicletSlicer1448559807354.chicletSlicerProps.header.textSize, defaultSettings.header.textSize);
                        defaultSettings.header.outline = powerbi.DataViewObjects.getValue(objects, ChicletSlicer1448559807354.chicletSlicerProps.header.outline, defaultSettings.header.outline);
                        defaultSettings.header.outlineColor = powerbi.DataViewObjects.getFillColor(objects, ChicletSlicer1448559807354.chicletSlicerProps.header.outlineColor, defaultSettings.header.outlineColor);
                        defaultSettings.header.outlineWeight = powerbi.DataViewObjects.getValue(objects, ChicletSlicer1448559807354.chicletSlicerProps.header.outlineWeight, defaultSettings.header.outlineWeight);
                        defaultSettings.slicerText.textSize = powerbi.DataViewObjects.getValue(objects, ChicletSlicer1448559807354.chicletSlicerProps.rows.textSize, defaultSettings.slicerText.textSize);
                        defaultSettings.slicerText.height = powerbi.DataViewObjects.getValue(objects, ChicletSlicer1448559807354.chicletSlicerProps.rows.height, defaultSettings.slicerText.height);
                        defaultSettings.slicerText.width = powerbi.DataViewObjects.getValue(objects, ChicletSlicer1448559807354.chicletSlicerProps.rows.width, defaultSettings.slicerText.width);
                        defaultSettings.slicerText.selectedColor = powerbi.DataViewObjects.getFillColor(objects, ChicletSlicer1448559807354.chicletSlicerProps.rows.selectedColor, defaultSettings.slicerText.selectedColor);
                        defaultSettings.slicerText.unselectedColor = powerbi.DataViewObjects.getFillColor(objects, ChicletSlicer1448559807354.chicletSlicerProps.rows.unselectedColor, defaultSettings.slicerText.unselectedColor);
                        defaultSettings.slicerText.disabledColor = powerbi.DataViewObjects.getFillColor(objects, ChicletSlicer1448559807354.chicletSlicerProps.rows.disabledColor, defaultSettings.slicerText.disabledColor);
                        defaultSettings.slicerText.background = powerbi.DataViewObjects.getFillColor(objects, ChicletSlicer1448559807354.chicletSlicerProps.rows.background, defaultSettings.slicerText.background);
                        defaultSettings.slicerText.transparency = powerbi.DataViewObjects.getValue(objects, ChicletSlicer1448559807354.chicletSlicerProps.rows.transparency, defaultSettings.slicerText.transparency);
                        defaultSettings.slicerText.fontColor = powerbi.DataViewObjects.getFillColor(objects, ChicletSlicer1448559807354.chicletSlicerProps.rows.fontColor, defaultSettings.slicerText.fontColor);
                        defaultSettings.slicerText.outline = powerbi.DataViewObjects.getValue(objects, ChicletSlicer1448559807354.chicletSlicerProps.rows.outline, defaultSettings.slicerText.outline);
                        defaultSettings.slicerText.outlineColor = powerbi.DataViewObjects.getFillColor(objects, ChicletSlicer1448559807354.chicletSlicerProps.rows.outlineColor, defaultSettings.slicerText.outlineColor);
                        defaultSettings.slicerText.outlineWeight = powerbi.DataViewObjects.getValue(objects, ChicletSlicer1448559807354.chicletSlicerProps.rows.outlineWeight, defaultSettings.slicerText.outlineWeight);
                        defaultSettings.slicerText.borderStyle = powerbi.DataViewObjects.getValue(objects, ChicletSlicer1448559807354.chicletSlicerProps.rows.borderStyle, defaultSettings.slicerText.borderStyle);
                        defaultSettings.images.imageSplit = powerbi.DataViewObjects.getValue(objects, ChicletSlicer1448559807354.chicletSlicerProps.images.imageSplit, defaultSettings.images.imageSplit);
                        defaultSettings.images.stretchImage = powerbi.DataViewObjects.getValue(objects, ChicletSlicer1448559807354.chicletSlicerProps.images.stretchImage, defaultSettings.images.stretchImage);
                        defaultSettings.images.bottomImage = powerbi.DataViewObjects.getValue(objects, ChicletSlicer1448559807354.chicletSlicerProps.images.bottomImage, defaultSettings.images.bottomImage);
                    }
                    var categories = dataView.categorical.categories[0];
                    slicerData = {
                        categorySourceName: categories.source.displayName,
                        formatString: visuals.valueFormatter.getFormatString(categories.source, ChicletSlicer1448559807354.chicletSlicerProps.formatString),
                        slicerSettings: defaultSettings,
                        slicerDataPoints: converter.dataPoints,
                    };
                    slicerData.hasSelectionOverride = converter.hasSelectionOverride;
                    return slicerData;
                };
                ChicletSlicer.prototype.init = function (options) {
                    this.element = options.element;
                    this.currentViewport = options.viewport;
                    if (this.behavior) {
                        this.interactivityService = visuals.createInteractivityService(options.host);
                    }
                    this.hostServices = options.host;
                    this.settings = ChicletSlicer.DefaultStyleProperties();
                    this.initContainer();
                };
                ChicletSlicer.prototype.update = function (options) {
                    if (!options ||
                        !options.dataViews ||
                        !options.dataViews[0] ||
                        !options.viewport) {
                        return;
                    }
                    var existingDataView = this.dataView;
                    this.dataView = options.dataViews[0];
                    var resetScrollbarPosition = true;
                    if (existingDataView) {
                        resetScrollbarPosition = !powerbi.DataViewAnalysis.hasSameCategoryIdentity(existingDataView, this.dataView);
                    }
                    if (options.viewport.height === this.currentViewport.height
                        && options.viewport.width === this.currentViewport.width) {
                        this.waitingForData = false;
                    }
                    else {
                        this.currentViewport = options.viewport;
                    }
                    this.updateInternal(resetScrollbarPosition);
                };
                ChicletSlicer.prototype.onResizing = function (finalViewport) {
                    this.currentViewport = finalViewport;
                    this.updateInternal(false);
                };
                ChicletSlicer.prototype.enumerateObjectInstances = function (options) {
                    var data = this.slicerData;
                    if (!data) {
                        return;
                    }
                    var objectName = options.objectName;
                    switch (objectName) {
                        case 'rows':
                            return this.enumerateRows(data);
                        case 'header':
                            return this.enumerateHeader(data);
                        case 'general':
                            return this.enumerateGeneral(data);
                        case 'images':
                            return this.enumerateImages(data);
                    }
                };
                ChicletSlicer.prototype.enumerateHeader = function (data) {
                    var slicerSettings = this.settings;
                    return [{
                            selector: null,
                            objectName: 'header',
                            properties: {
                                show: slicerSettings.header.show,
                                title: slicerSettings.header.title,
                                fontColor: slicerSettings.header.fontColor,
                                background: slicerSettings.header.background,
                                textSize: slicerSettings.header.textSize,
                                outline: slicerSettings.header.outline,
                                outlineColor: slicerSettings.header.outlineColor,
                                outlineWeight: slicerSettings.header.outlineWeight
                            }
                        }];
                };
                ChicletSlicer.prototype.enumerateRows = function (data) {
                    var slicerSettings = this.settings;
                    return [{
                            selector: null,
                            objectName: 'rows',
                            properties: {
                                textSize: slicerSettings.slicerText.textSize,
                                height: slicerSettings.slicerText.height,
                                width: slicerSettings.slicerText.width,
                                background: slicerSettings.slicerText.background,
                                transparency: slicerSettings.slicerText.transparency,
                                selectedColor: slicerSettings.slicerText.selectedColor,
                                unselectedColor: slicerSettings.slicerText.unselectedColor,
                                disabledColor: slicerSettings.slicerText.disabledColor,
                                outline: slicerSettings.slicerText.outline,
                                outlineColor: slicerSettings.slicerText.outlineColor,
                                outlineWeight: slicerSettings.slicerText.outlineWeight,
                                fontColor: slicerSettings.slicerText.fontColor,
                                borderStyle: slicerSettings.slicerText.borderStyle,
                            }
                        }];
                };
                ChicletSlicer.prototype.enumerateGeneral = function (data) {
                    var slicerSettings = this.settings;
                    return [{
                            selector: null,
                            objectName: 'general',
                            properties: {
                                orientation: slicerSettings.general.orientation,
                                columns: slicerSettings.general.columns,
                                rows: slicerSettings.general.rows,
                                showDisabled: slicerSettings.general.showDisabled,
                                multiselect: slicerSettings.general.multiselect,
                            }
                        }];
                };
                ChicletSlicer.prototype.enumerateImages = function (data) {
                    var slicerSettings = this.settings;
                    return [{
                            selector: null,
                            objectName: 'images',
                            properties: {
                                imageSplit: slicerSettings.images.imageSplit,
                                stretchImage: slicerSettings.images.stretchImage,
                                bottomImage: slicerSettings.images.bottomImage,
                            }
                        }];
                };
                ChicletSlicer.prototype.updateInternal = function (resetScrollbarPosition) {
                    this.updateSlicerBodyDimensions();
                    var localizedSelectAllText = 'Select All';
                    var data = ChicletSlicer.converter(this.dataView, localizedSelectAllText, this.interactivityService);
                    if (!data) {
                        this.tableView.empty();
                        return;
                    }
                    data.slicerSettings.header.outlineWeight = data.slicerSettings.header.outlineWeight < 0 ? 0 : data.slicerSettings.header.outlineWeight;
                    this.slicerData = data;
                    this.settings = this.slicerData.slicerSettings;
                    if (this.settings.general.showDisabled === ChicletSlicerShowDisabled.BOTTOM) {
                        data.slicerDataPoints.sort(function (a, b) {
                            if (a.selectable === b.selectable) {
                                return 0;
                            }
                            else if (a.selectable && !b.selectable) {
                                return -1;
                            }
                            else {
                                return 1;
                            }
                        });
                    }
                    else if (this.settings.general.showDisabled === ChicletSlicerShowDisabled.HIDE) {
                        data.slicerDataPoints = data.slicerDataPoints.filter(function (x) { return x.selectable; });
                    }
                    var textProperties = ChicletSlicer.getChicletTextProperties(this.settings.slicerText.textSize);
                    var height = this.settings.slicerText.height > 0 ? this.settings.slicerText.height :
                        (data.slicerDataPoints.length === 0 || data.slicerDataPoints[0].imageURL !== '' ? 100 :
                            powerbi.TextMeasurementService.estimateSvgTextHeight(textProperties) + ChicletSlicerTextMeasurementHelper.estimateSvgTextBaselineDelta(textProperties));
                    this.tableView
                        .rowHeight(height)
                        .columnWidth(this.settings.slicerText.width)
                        .orientation(this.settings.general.orientation)
                        .rows(this.settings.general.rows)
                        .columns(this.settings.general.columns)
                        .data(data.slicerDataPoints, function (d) { return $.inArray(d, data.slicerDataPoints); }, resetScrollbarPosition)
                        .viewport(this.getSlicerBodyViewport(this.currentViewport))
                        .render();
                };
                ChicletSlicer.prototype.initContainer = function () {
                    var _this = this;
                    var settings = this.settings;
                    var slicerBodyViewport = this.getSlicerBodyViewport(this.currentViewport);
                    var slicerContainer = d3.select(this.element.get(0))
                        .append('div')
                        .classed(ChicletSlicer.Container.class, true);
                    this.slicerHeader = slicerContainer
                        .append('div')
                        .classed(ChicletSlicer.Header.class, true);
                    this.slicerHeader
                        .append('span')
                        .classed(ChicletSlicer.Clear.class, true)
                        .attr('title', 'Clear');
                    this.slicerHeader
                        .append('div')
                        .classed(ChicletSlicer.HeaderText.class, true)
                        .style({
                        'margin-left': PixelConverter.toString(settings.headerText.marginLeft),
                        'margin-top': PixelConverter.toString(settings.headerText.marginTop),
                        'border-style': this.getBorderStyle(settings.header.outline),
                        'border-color': settings.header.outlineColor,
                        'border-width': this.getBorderWidth(settings.header.outline, settings.header.outlineWeight),
                        'font-size': PixelConverter.fromPoint(settings.header.textSize),
                    });
                    this.slicerBody = slicerContainer
                        .append('div').classed(ChicletSlicer.Body.class, true)
                        .classed('slicerBody-horizontal', settings.general.orientation === Orientation.HORIZONTAL)
                        .style({
                        'height': PixelConverter.toString(slicerBodyViewport.height),
                        'width': '100%',
                    });
                    var rowEnter = function (rowSelection) {
                        var settings = _this.settings;
                        var listItemElement = rowSelection.append('li')
                            .classed(ChicletSlicer.ItemContainer.class, true)
                            .style({
                            'margin-left': PixelConverter.toString(settings.slicerItemContainer.marginLeft),
                        });
                        listItemElement.append('div')
                            .classed('slicer-img-wrapper', true);
                        listItemElement.append('div')
                            .classed('slicer-text-wrapper', true)
                            .append('span')
                            .classed(ChicletSlicer.LabelText.class, true)
                            .style({
                            'font-size': PixelConverter.fromPoint(settings.slicerText.textSize),
                        });
                    };
                    var rowUpdate = function (rowSelection) {
                        var settings = _this.settings;
                        var data = _this.slicerData;
                        if (data && settings) {
                            _this.slicerHeader.classed('hidden', !settings.header.show);
                            _this.slicerHeader.select(ChicletSlicer.HeaderText.selector)
                                .text(settings.header.title.trim() !== "" ? settings.header.title.trim() : _this.slicerData.categorySourceName)
                                .style({
                                'border-style': _this.getBorderStyle(settings.header.outline),
                                'border-color': settings.header.outlineColor,
                                'border-width': _this.getBorderWidth(settings.header.outline, settings.header.outlineWeight),
                                'color': settings.header.fontColor,
                                'background-color': settings.header.background,
                                'font-size': PixelConverter.fromPoint(settings.header.textSize),
                            });
                            _this.slicerBody
                                .classed('slicerBody-horizontal', settings.general.orientation === Orientation.HORIZONTAL);
                            var slicerText = rowSelection.selectAll(ChicletSlicer.LabelText.selector);
                            var textProperties = ChicletSlicer.getChicletTextProperties(settings.slicerText.textSize);
                            var formatString = data.formatString;
                            slicerText.text(function (d) {
                                var text = visuals.valueFormatter.format(d.category, formatString);
                                textProperties.text = text;
                                return powerbi.TextMeasurementService.getTailoredTextOrDefault(textProperties, _this.currentViewport.width / _this.settings.general.columns);
                            });
                            var slicerImg = rowSelection.selectAll('.slicer-img-wrapper');
                            slicerImg
                                .style('height', settings.images.imageSplit + '%')
                                .classed('hidden', function (d) {
                                if (!(d.imageURL)) {
                                    return true;
                                }
                                if (settings.images.imageSplit < 10) {
                                    return true;
                                }
                            })
                                .style('display', function (d) { return (d.imageURL) ? 'flex' : 'none'; })
                                .classed('stretchImage', settings.images.stretchImage)
                                .classed('bottomImage', settings.images.bottomImage)
                                .style('background-image', function (d) {
                                return d.imageURL ? "url(" + d.imageURL + ")" : '';
                            });
                            rowSelection.selectAll('.slicer-text-wrapper')
                                .style('height', function (d) {
                                return d.imageURL ? (100 - settings.images.imageSplit) + '%' : '100%';
                            })
                                .classed('hidden', function (d) {
                                if (settings.images.imageSplit > 90) {
                                    return true;
                                }
                            });
                            rowSelection.selectAll('.slicerItemContainer').style({
                                'color': settings.slicerText.fontColor,
                                'border-style': _this.getBorderStyle(settings.slicerText.outline),
                                'border-color': settings.slicerText.outlineColor,
                                'border-width': _this.getBorderWidth(settings.slicerText.outline, settings.slicerText.outlineWeight),
                                'font-size': PixelConverter.fromPoint(settings.slicerText.textSize),
                                'border-radius': _this.getBorderRadius(settings.slicerText.borderStyle),
                            });
                            _this.slicerBody.style('background-color', explore.util.hexToRGBString(settings.slicerText.background, (100 - settings.slicerText.transparency) / 100));
                            if (_this.interactivityService && _this.slicerBody) {
                                var slicerBody = _this.slicerBody.attr('width', _this.currentViewport.width);
                                var slicerItemContainers = slicerBody.selectAll(ChicletSlicer.ItemContainer.selector);
                                var slicerItemLabels = slicerBody.selectAll(ChicletSlicer.LabelText.selector);
                                var slicerItemInputs = slicerBody.selectAll(ChicletSlicer.Input.selector);
                                var slicerClear = _this.slicerHeader.select(ChicletSlicer.Clear.selector);
                                var behaviorOptions = {
                                    dataPoints: data.slicerDataPoints,
                                    slicerItemContainers: slicerItemContainers,
                                    slicerItemLabels: slicerItemLabels,
                                    slicerItemInputs: slicerItemInputs,
                                    slicerClear: slicerClear,
                                    interactivityService: _this.interactivityService,
                                    slicerSettings: data.slicerSettings,
                                };
                                _this.interactivityService.bind(data.slicerDataPoints, _this.behavior, behaviorOptions, {
                                    overrideSelectionFromData: true,
                                    hasSelectionOverride: data.hasSelectionOverride
                                });
                                _this.behavior.styleSlicerInputs(rowSelection.select(ChicletSlicer.ItemContainer.selector), _this.interactivityService.hasSelection());
                            }
                            else {
                                _this.behavior.styleSlicerInputs(rowSelection.select(ChicletSlicer.ItemContainer.selector), false);
                            }
                        }
                    };
                    var rowExit = function (rowSelection) {
                        rowSelection.remove();
                    };
                    var tableViewOptions = {
                        rowHeight: this.getRowHeight(),
                        columnWidth: this.settings.slicerText.width,
                        orientation: this.settings.general.orientation,
                        rows: this.settings.general.rows,
                        columns: this.settings.general.columns,
                        enter: rowEnter,
                        exit: rowExit,
                        update: rowUpdate,
                        loadMoreData: function () { return _this.onLoadMoreData(); },
                        scrollEnabled: true,
                        viewport: this.getSlicerBodyViewport(this.currentViewport),
                        baseContainer: this.slicerBody,
                    };
                    this.tableView = TableViewFactory.createTableView(tableViewOptions);
                };
                ChicletSlicer.prototype.onLoadMoreData = function () {
                    if (!this.waitingForData && this.dataView.metadata && this.dataView.metadata.segment) {
                        this.hostServices.loadMoreData();
                        this.waitingForData = true;
                    }
                };
                ChicletSlicer.prototype.getSlicerBodyViewport = function (currentViewport) {
                    var settings = this.settings;
                    var headerHeight = (settings.header.show) ? this.getHeaderHeight() : 0;
                    var slicerBodyHeight = currentViewport.height - (headerHeight + settings.header.borderBottomWidth);
                    return {
                        height: slicerBodyHeight,
                        width: currentViewport.width
                    };
                };
                ChicletSlicer.prototype.updateSlicerBodyDimensions = function () {
                    var slicerViewport = this.getSlicerBodyViewport(this.currentViewport);
                    this.slicerBody
                        .style({
                        'height': PixelConverter.toString(slicerViewport.height),
                        'width': '100%',
                    });
                };
                ChicletSlicer.getChicletTextProperties = function (textSize) {
                    return {
                        fontFamily: ChicletSlicer.DefaultFontFamily,
                        fontSize: PixelConverter.fromPoint(textSize || ChicletSlicer.DefaultFontSizeInPt),
                    };
                };
                ChicletSlicer.prototype.getHeaderHeight = function () {
                    return powerbi.TextMeasurementService.estimateSvgTextHeight(ChicletSlicer.getChicletTextProperties(this.settings.header.textSize));
                };
                ChicletSlicer.prototype.getRowHeight = function () {
                    var textSettings = this.settings.slicerText;
                    return textSettings.height !== 0
                        ? textSettings.height
                        : powerbi.TextMeasurementService.estimateSvgTextHeight(ChicletSlicer.getChicletTextProperties(textSettings.textSize));
                };
                ChicletSlicer.prototype.getBorderStyle = function (outlineElement) {
                    return outlineElement === '0px' ? 'none' : 'solid';
                };
                ChicletSlicer.prototype.getBorderWidth = function (outlineElement, outlineWeight) {
                    switch (outlineElement) {
                        case 'None':
                            return '0px';
                        case 'BottomOnly':
                            return '0px 0px ' + outlineWeight + 'px 0px';
                        case 'TopOnly':
                            return outlineWeight + 'px 0px 0px 0px';
                        case 'TopBottom':
                            return outlineWeight + 'px 0px ' + outlineWeight + 'px 0px';
                        case 'LeftRight':
                            return '0px ' + outlineWeight + 'px 0px ' + outlineWeight + 'px';
                        case 'Frame':
                            return outlineWeight + 'px';
                        default:
                            return outlineElement.replace("1", outlineWeight.toString());
                    }
                };
                ChicletSlicer.prototype.getBorderRadius = function (borderType) {
                    switch (borderType) {
                        case ChicletBorderStyle.ROUNDED:
                            return "10px";
                        case ChicletBorderStyle.SQUARE:
                            return "0px";
                        default:
                            return "5px";
                    }
                };
                ChicletSlicer.capabilities = {
                    dataRoles: [
                        {
                            name: 'Category',
                            kind: powerbi.VisualDataRoleKind.Grouping,
                            displayName: powerbi.data.createDisplayNameGetter('Role_DisplayName_Field'),
                        },
                        {
                            name: 'Values',
                            kind: powerbi.VisualDataRoleKind.Measure,
                            displayName: powerbi.data.createDisplayNameGetter('Role_DisplayName_Values'),
                        },
                        {
                            name: 'Image',
                            kind: powerbi.VisualDataRoleKind.Grouping,
                            displayName: 'Image',
                        },
                    ],
                    objects: {
                        general: {
                            displayName: powerbi.data.createDisplayNameGetter('Visual_General'),
                            properties: {
                                orientation: {
                                    displayName: 'Orientation',
                                    type: { enumeration: Orientation.type }
                                },
                                columns: {
                                    displayName: 'Columns',
                                    type: { numeric: true }
                                },
                                rows: {
                                    displayName: 'Rows',
                                    type: { numeric: true }
                                },
                                showDisabled: {
                                    displayName: 'Show Disabled',
                                    type: { enumeration: ChicletSlicerShowDisabled.type }
                                },
                                multiselect: {
                                    displayName: 'Multiple selection',
                                    type: { bool: true }
                                },
                                selected: {
                                    type: { bool: true }
                                },
                                filter: {
                                    type: { filter: {} },
                                    rule: {
                                        output: {
                                            property: 'selected',
                                            selector: ['Category'],
                                        }
                                    }
                                },
                                formatString: {
                                    type: { formatting: { formatString: true } },
                                },
                            },
                        },
                        header: {
                            displayName: powerbi.data.createDisplayNameGetter('Visual_Header'),
                            properties: {
                                show: {
                                    displayName: powerbi.data.createDisplayNameGetter('Visual_Show'),
                                    type: { bool: true }
                                },
                                title: {
                                    displayName: 'Title',
                                    type: { text: true }
                                },
                                fontColor: {
                                    displayName: powerbi.data.createDisplayNameGetter('Visual_FontColor'),
                                    type: { fill: { solid: { color: true } } }
                                },
                                background: {
                                    displayName: powerbi.data.createDisplayNameGetter('Visual_Background'),
                                    type: { fill: { solid: { color: true } } }
                                },
                                outline: {
                                    displayName: powerbi.data.createDisplayNameGetter('Visual_Outline'),
                                    type: { formatting: { outline: true } }
                                },
                                textSize: {
                                    displayName: powerbi.data.createDisplayNameGetter('Visual_TextSize'),
                                    type: { numeric: true }
                                },
                                outlineColor: {
                                    displayName: 'Outline Color',
                                    type: { fill: { solid: { color: true } } }
                                },
                                outlineWeight: {
                                    displayName: 'Outline Weight',
                                    type: { numeric: true }
                                }
                            }
                        },
                        rows: {
                            displayName: 'Chiclets',
                            properties: {
                                fontColor: {
                                    displayName: 'Text color',
                                    type: { fill: { solid: { color: true } } }
                                },
                                textSize: {
                                    displayName: powerbi.data.createDisplayNameGetter('Visual_TextSize'),
                                    type: { numeric: true }
                                },
                                height: {
                                    displayName: 'Height',
                                    type: { numeric: true }
                                },
                                width: {
                                    displayName: 'Width',
                                    type: { numeric: true }
                                },
                                selectedColor: {
                                    displayName: 'Selected Color',
                                    type: { fill: { solid: { color: true } } }
                                },
                                unselectedColor: {
                                    displayName: 'Unselected Color',
                                    type: { fill: { solid: { color: true } } }
                                },
                                disabledColor: {
                                    displayName: 'Disabled Color',
                                    type: { fill: { solid: { color: true } } }
                                },
                                background: {
                                    displayName: powerbi.data.createDisplayNameGetter('Visual_Background'),
                                    type: { fill: { solid: { color: true } } }
                                },
                                transparency: {
                                    displayName: "Transparency",
                                    description: "Set transparency for background color",
                                    type: { numeric: true }
                                },
                                outline: {
                                    displayName: powerbi.data.createDisplayNameGetter('Visual_Outline'),
                                    type: { formatting: { outline: true } }
                                },
                                outlineColor: {
                                    displayName: 'Outline Color',
                                    type: { fill: { solid: { color: true } } }
                                },
                                outlineWeight: {
                                    displayName: 'Outline Weight',
                                    type: { numeric: true }
                                },
                                borderStyle: {
                                    displayName: 'Outline Style',
                                    type: { enumeration: ChicletBorderStyle.type }
                                },
                            }
                        },
                        images: {
                            displayName: 'Images',
                            properties: {
                                imageSplit: {
                                    displayName: 'Image Split',
                                    type: { numeric: true }
                                },
                                stretchImage: {
                                    displayName: 'Stretch image',
                                    type: { bool: true }
                                },
                                bottomImage: {
                                    displayName: 'Bottom image',
                                    type: { bool: true }
                                },
                            }
                        },
                    },
                    dataViewMappings: [{
                            conditions: [
                                { 'Category': { max: 1 }, 'Image': { min: 0, max: 1 }, 'Values': { min: 0, max: 1 } }],
                            categorical: {
                                categories: {
                                    for: { in: 'Category' },
                                    dataReductionAlgorithm: { top: {} }
                                },
                                values: {
                                    group: {
                                        by: 'Image',
                                        select: [{ bind: { to: 'Values' } },
                                        ],
                                        dataReductionAlgorithm: { top: {} }
                                    }
                                },
                                includeEmptyGroups: true
                            }
                        }],
                    supportsHighlight: true,
                    sorting: {
                        default: {},
                    },
                    suppressDefaultTitle: true,
                };
                ChicletSlicer.DefaultFontFamily = 'Segoe UI, Tahoma, Verdana, Geneva, sans-serif';
                ChicletSlicer.DefaultFontSizeInPt = 11;
                ChicletSlicer.ItemContainer = createClassAndSelector('slicerItemContainer');
                ChicletSlicer.HeaderText = createClassAndSelector('headerText');
                ChicletSlicer.Container = createClassAndSelector('chicletSlicer');
                ChicletSlicer.LabelText = createClassAndSelector('slicerText');
                ChicletSlicer.Header = createClassAndSelector('slicerHeader');
                ChicletSlicer.Input = createClassAndSelector('slicerCheckbox');
                ChicletSlicer.Clear = createClassAndSelector('clear');
                ChicletSlicer.Body = createClassAndSelector('slicerBody');
                return ChicletSlicer;
            })();
            ChicletSlicer1448559807354.ChicletSlicer = ChicletSlicer;
            var ChicletSlicerChartConversion;
            (function (ChicletSlicerChartConversion) {
                var ChicletSlicerConverter = (function () {
                    function ChicletSlicerConverter(dataView, interactivityService) {
                        var dataViewCategorical = dataView.categorical;
                        this.dataViewCategorical = dataViewCategorical;
                        this.dataViewMetadata = dataView.metadata;
                        if (dataViewCategorical.categories && dataViewCategorical.categories.length > 0) {
                            this.category = dataViewCategorical.categories[0];
                            this.categoryIdentities = this.category.identity;
                            this.categoryValues = this.category.values;
                            this.categoryColumnRef = this.category.identityFields;
                            this.categoryFormatString = visuals.valueFormatter.getFormatString(this.category.source, ChicletSlicer1448559807354.chicletSlicerProps.formatString);
                        }
                        this.dataPoints = [];
                        this.interactivityService = interactivityService;
                        this.hasSelectionOverride = false;
                    }
                    ChicletSlicerConverter.prototype.convert = function () {
                        this.dataPoints = [];
                        this.numberOfCategoriesSelectedInData = 0;
                        if (this.categoryValues) {
                            var objects = this.dataViewMetadata ? this.dataViewMetadata.objects : undefined;
                            var isInvertedSelectionMode = undefined;
                            var numberOfScopeIds;
                            if (objects && objects.general && objects.general.filter) {
                                if (!this.categoryColumnRef)
                                    return;
                                var filter = objects.general.filter;
                                var scopeIds = powerbi.data.SQExprConverter.asScopeIdsContainer(filter, this.categoryColumnRef);
                                if (scopeIds) {
                                    isInvertedSelectionMode = scopeIds.isNot;
                                    numberOfScopeIds = scopeIds.scopeIds ? scopeIds.scopeIds.length : 0;
                                }
                                else {
                                    isInvertedSelectionMode = false;
                                }
                            }
                            if (this.interactivityService) {
                                if (isInvertedSelectionMode === undefined) {
                                    isInvertedSelectionMode = this.interactivityService.isSelectionModeInverted();
                                }
                                else {
                                    this.interactivityService.setSelectionModeInverted(isInvertedSelectionMode);
                                }
                            }
                            var hasSelection = undefined;
                            for (var idx = 0; idx < this.categoryValues.length; idx++) {
                                var selected = visuals.isCategoryColumnSelected(ChicletSlicer1448559807354.chicletSlicerProps.selectedPropertyIdentifier, this.category, idx);
                                if (selected != null) {
                                    hasSelection = selected;
                                    break;
                                }
                            }
                            var dataViewCategorical = this.dataViewCategorical;
                            var formatStringProp = ChicletSlicer1448559807354.chicletSlicerProps.formatString;
                            var value = -Infinity;
                            var imageURL = '';
                            for (var categoryIndex = 0, categoryCount = this.categoryValues.length; categoryIndex < categoryCount; categoryIndex++) {
                                var categoryIdentity = this.category.identity ? this.category.identity[categoryIndex] : null;
                                var categoryIsSelected = visuals.isCategoryColumnSelected(ChicletSlicer1448559807354.chicletSlicerProps.selectedPropertyIdentifier, this.category, categoryIndex);
                                var selectable = true;
                                if (hasSelection != null) {
                                    if (isInvertedSelectionMode) {
                                        if (this.category.objects == null)
                                            categoryIsSelected = undefined;
                                        if (categoryIsSelected != null) {
                                            categoryIsSelected = hasSelection;
                                        }
                                        else if (categoryIsSelected == null)
                                            categoryIsSelected = !hasSelection;
                                    }
                                    else {
                                        if (categoryIsSelected == null) {
                                            categoryIsSelected = !hasSelection;
                                        }
                                    }
                                }
                                if (categoryIsSelected) {
                                    this.numberOfCategoriesSelectedInData++;
                                }
                                var categoryValue = this.categoryValues[categoryIndex];
                                var categoryLabel = visuals.valueFormatter.format(categoryValue, this.categoryFormatString);
                                if (this.dataViewCategorical.values) {
                                    for (var seriesIndex = 0; seriesIndex < this.dataViewCategorical.values.length; seriesIndex++) {
                                        var seriesData = dataViewCategorical.values[seriesIndex];
                                        if (seriesData.values[categoryIndex] != null) {
                                            value = seriesData.values[categoryIndex];
                                            if (seriesData.highlights) {
                                                selectable = !(seriesData.highlights[categoryIndex] === null);
                                            }
                                            if (seriesData.source.groupName && seriesData.source.groupName !== '') {
                                                imageURL = visuals.converterHelper.getFormattedLegendLabel(seriesData.source, dataViewCategorical.values, formatStringProp);
                                                if (!/^(ftp|http|https):\/\/[^ "]+$/.test(imageURL)) {
                                                    imageURL = undefined;
                                                }
                                            }
                                        }
                                    }
                                }
                                this.dataPoints.push({
                                    identity: visuals.SelectionId.createWithId(categoryIdentity),
                                    category: categoryLabel,
                                    imageURL: imageURL,
                                    value: value,
                                    selected: categoryIsSelected,
                                    selectable: selectable
                                });
                            }
                            if (numberOfScopeIds != null && numberOfScopeIds > this.numberOfCategoriesSelectedInData) {
                                this.hasSelectionOverride = true;
                            }
                        }
                    };
                    return ChicletSlicerConverter;
                })();
                ChicletSlicerChartConversion.ChicletSlicerConverter = ChicletSlicerConverter;
            })(ChicletSlicerChartConversion || (ChicletSlicerChartConversion = {}));
            var ChicletSlicerTextMeasurementHelper;
            (function (ChicletSlicerTextMeasurementHelper) {
                var spanElement;
                var svgTextElement;
                var canvasCtx;
                function estimateSvgTextBaselineDelta(textProperties) {
                    var rect = estimateSvgTextRect(textProperties);
                    return rect.y + rect.height;
                }
                ChicletSlicerTextMeasurementHelper.estimateSvgTextBaselineDelta = estimateSvgTextBaselineDelta;
                function ensureDOM() {
                    if (spanElement)
                        return;
                    spanElement = $('<span/>');
                    $('body').append(spanElement);
                    svgTextElement = d3.select($('body').get(0))
                        .append('svg')
                        .style({
                        'height': '0px',
                        'width': '0px',
                        'position': 'absolute'
                    })
                        .append('text');
                    canvasCtx = $('<canvas/>').get(0).getContext("2d");
                }
                function measureSvgTextRect(textProperties) {
                    debug.assertValue(textProperties, 'textProperties');
                    ensureDOM();
                    svgTextElement.style(null);
                    svgTextElement
                        .text(textProperties.text)
                        .attr({
                        'visibility': 'hidden',
                        'font-family': textProperties.fontFamily,
                        'font-size': textProperties.fontSize,
                        'font-weight': textProperties.fontWeight,
                        'font-style': textProperties.fontStyle,
                        'white-space': textProperties.whiteSpace || 'nowrap'
                    });
                    return svgTextElement.node().getBBox();
                }
                function estimateSvgTextRect(textProperties) {
                    debug.assertValue(textProperties, 'textProperties');
                    var estimatedTextProperties = {
                        fontFamily: textProperties.fontFamily,
                        fontSize: textProperties.fontSize,
                        text: "M",
                    };
                    var rect = measureSvgTextRect(estimatedTextProperties);
                    return rect;
                }
            })(ChicletSlicerTextMeasurementHelper = ChicletSlicer1448559807354.ChicletSlicerTextMeasurementHelper || (ChicletSlicer1448559807354.ChicletSlicerTextMeasurementHelper = {}));
            var ChicletSlicerWebBehavior = (function () {
                function ChicletSlicerWebBehavior() {
                }
                ChicletSlicerWebBehavior.prototype.bindEvents = function (options, selectionHandler) {
                    var _this = this;
                    var filterPropertyId = ChicletSlicer1448559807354.chicletSlicerProps.filterPropertyIdentifier;
                    var slicers = this.slicers = options.slicerItemContainers;
                    this.slicerItemLabels = options.slicerItemLabels;
                    this.slicerItemInputs = options.slicerItemInputs;
                    var slicerClear = options.slicerClear;
                    this.dataPoints = options.dataPoints;
                    this.interactivityService = options.interactivityService;
                    this.slicerSettings = options.slicerSettings;
                    slicers.on("mouseover", function (d) {
                        if (d.selectable) {
                            d.mouseOver = true;
                            d.mouseOut = false;
                            _this.renderMouseover();
                        }
                    });
                    slicers.on("mouseout", function (d) {
                        if (d.selectable) {
                            d.mouseOver = false;
                            d.mouseOut = true;
                            _this.renderMouseover();
                        }
                    });
                    slicers.on("click", function (d, index) {
                        if (!d.selectable) {
                            return;
                        }
                        var settings = _this.slicerSettings;
                        d3.event.preventDefault();
                        if (d3.event.altKey && settings.general.multiselect) {
                            var selectedIndexes = jQuery.map(_this.dataPoints, function (d, index) { if (d.selected)
                                return index; });
                            var selIndex = selectedIndexes.length > 0 ? (selectedIndexes[selectedIndexes.length - 1]) : 0;
                            if (selIndex > index) {
                                var temp = index;
                                index = selIndex;
                                selIndex = temp;
                            }
                            selectionHandler.handleClearSelection();
                            for (var i = selIndex; i <= index; i++) {
                                selectionHandler.handleSelection(_this.dataPoints[i], true);
                            }
                        }
                        else if (d3.event.ctrlKey && settings.general.multiselect) {
                            selectionHandler.handleSelection(d, true);
                        }
                        else {
                            selectionHandler.handleSelection(d, false);
                        }
                        selectionHandler.persistSelectionFilter(filterPropertyId);
                    });
                    slicerClear.on("click", function (d) {
                        selectionHandler.handleClearSelection();
                        selectionHandler.persistSelectionFilter(filterPropertyId);
                    });
                };
                ChicletSlicerWebBehavior.prototype.renderSelection = function (hasSelection) {
                    if (!hasSelection && !this.interactivityService.isSelectionModeInverted()) {
                        this.slicers.style('background', this.slicerSettings.slicerText.unselectedColor);
                    }
                    else {
                        this.styleSlicerInputs(this.slicers, hasSelection);
                    }
                };
                ChicletSlicerWebBehavior.prototype.renderMouseover = function () {
                    var _this = this;
                    this.slicerItemLabels.style({
                        'color': function (d) {
                            if (d.mouseOver)
                                return _this.slicerSettings.slicerText.hoverColor;
                            if (d.mouseOut) {
                                if (d.selected)
                                    return _this.slicerSettings.slicerText.fontColor;
                                else
                                    return _this.slicerSettings.slicerText.fontColor;
                            }
                        }
                    });
                };
                ChicletSlicerWebBehavior.prototype.styleSlicerInputs = function (slicers, hasSelection) {
                    var settings = this.slicerSettings;
                    slicers.each(function (d) {
                        d3.select(this).style({
                            'background': d.selectable ? (d.selected ? settings.slicerText.selectedColor : settings.slicerText.unselectedColor)
                                : settings.slicerText.disabledColor
                        });
                        d3.select(this).classed('slicerItem-disabled', !d.selectable);
                    });
                };
                return ChicletSlicerWebBehavior;
            })();
            ChicletSlicer1448559807354.ChicletSlicerWebBehavior = ChicletSlicerWebBehavior;
            var explore;
            (function (explore) {
                var util;
                (function (util) {
                    function hexToRGBString(hex, transparency) {
                        var shorthandRegex = /^#?([a-f\d])([a-f\d])([a-f\d])$/i;
                        hex = hex.replace(shorthandRegex, function (m, r, g, b) {
                            return r + r + g + g + b + b;
                        });
                        var result = /^#?([a-f\d]{2})([a-f\d]{2})([a-f\d]{2})$/i.exec(hex);
                        var rgb = result ? {
                            r: parseInt(result[1], 16),
                            g: parseInt(result[2], 16),
                            b: parseInt(result[3], 16)
                        } : null;
                        if (rgb === null) {
                            return '';
                        }
                        if (!transparency && transparency !== 0) {
                            return "rgb(" + rgb.r + "," + rgb.g + "," + rgb.b + ")";
                        }
                        else {
                            return "rgba(" + rgb.r + "," + rgb.g + "," + rgb.b + "," + transparency + ")";
                        }
                    }
                    util.hexToRGBString = hexToRGBString;
                })(util = explore.util || (explore.util = {}));
            })(explore || (explore = {}));
        })(ChicletSlicer1448559807354 = visuals.ChicletSlicer1448559807354 || (visuals.ChicletSlicer1448559807354 = {}));
    })(visuals = powerbi.visuals || (powerbi.visuals = {}));
})(powerbi || (powerbi = {}));

var powerbi;
(function (powerbi) {
    var visuals;
    (function (visuals) {
        var plugins;
        (function (plugins) {
            plugins.ChicletSlicer1448559807354 = {
                name: 'ChicletSlicer1448559807354',
                class: 'ChicletSlicer1448559807354',
                capabilities: powerbi.visuals.ChicletSlicer1448559807354.ChicletSlicer.capabilities,
                custom: true,
                create: function () { return new powerbi.visuals.ChicletSlicer1448559807354.ChicletSlicer(); }
            };
        })(plugins = visuals.plugins || (visuals.plugins = {}));
    })(visuals = powerbi.visuals || (powerbi.visuals = {}));
})(powerbi || (powerbi = {}));

//# sourceURL=ChicletSlicer.js
//# sourceMappingURL=ChicletSlicer1448559807354.js.map