/*
 *  Power BI Visualizations
 *
 *  Copyright (c) Microsoft Corporation
 *  All rights reserved.
 *  MIT License
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the ""Software""), to deal
 *  in the Software without restriction, including without limitation the rights
 *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED *AS IS*, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 *  THE SOFTWARE.
 */
// -- /// <reference path="../../_references.ts"></reference>
var powerbi;
(function (powerbi) {
    var visuals;
    (function (visuals) {
        var Sunburst1445472000808;
        (function (Sunburst1445472000808) {
            var SelectionManager = visuals.utility.SelectionManager;
            Sunburst1445472000808.sunburstRoleNames = {
                nodes: 'Nodes',
                values: 'Values',
            };
            var Sunburst = (function () {
                function Sunburst() {
                    this.total = 0;
                }
                Sunburst.prototype.init = function (options) {
                    var _this = this;
                    this.arc = d3.svg.arc().startAngle(function (d) {
                        return d.x;
                    }).endAngle(function (d) {
                        return d.x + d.dx;
                    }).innerRadius(function (d) {
                        return Math.sqrt(d.y);
                    }).outerRadius(function (d) {
                        return Math.sqrt(d.y + d.dy);
                    });
                    this.colors = options.style.colorPalette.dataColors;
                    this.selectionManager = new SelectionManager({ hostServices: options.host });
                    this.svg = d3.select(options.element.get(0)).append('svg');
                    this.svg.classed('mainDrawArea', true);
                    this.g = this.svg.append('g');
                    this.g.classed("container", true);
                    this.svg.append("text").classed("sunBurstPercentageFixed", true);
                    this.svg.on('mousedown', function (d) {
                        _this.svg.selectAll("path").style("opacity", 1);
                        _this.svg.select(".sunBurstPercentageFixed").style("opacity", 0);
                        _this.selectionManager.clear();
                    });
                };
                Sunburst.setAllUnhide = function (selection) {
                    selection.attr("setUnHide", "true");
                };
                Sunburst.prototype.update = function (options) {
                    if (options.dataViews.length > 0) {
                        var data = this.converter(options.dataViews[0], this.colors);
                        this.viewport = options.viewport;
                        this.updateInternal(data);
                    }
                };
                Sunburst.prototype.updateInternal = function (dataRootNode) {
                    var _this = this;
                    this.svg.attr({
                        'height': this.viewport.height,
                        'width': this.viewport.width
                    });
                    this.g.attr('transform', visuals.SVGUtil.translate(this.viewport.width / 2, this.viewport.height / 2));
                    var radius = Math.min(this.viewport.width, this.viewport.height) / 2;
                    var partition = d3.layout.partition().size([2 * Math.PI, radius * radius]).value(function (d) {
                        return d.value;
                    });
                    var path = this.g.datum(dataRootNode).selectAll("path").data(partition.nodes);
                    path.enter().append("path");
                    path.attr("display", function (d) {
                        return d.depth ? null : "none";
                    }).attr("d", this.arc).style("stroke", "#fff").style("fill", function (d) {
                        return d.color;
                    }).style("fill-rule", "evenodd").on("mousedown", function (d) {
                        if (d.selector) {
                            _this.selectionManager.select(d.selector);
                        }
                        d3.selectAll("path").call(Sunburst.setAllUnhide).attr('setUnHide', null);
                        _this.highlightPath(d, _this, true);
                        var percentageFixedText = _this.svg.select(".sunBurstPercentageFixed");
                        var percentage = _this.total === 0 ? 0 : (100 * d.total / _this.total).toPrecision(3);
                        percentageFixedText.text(d ? percentage + "%" : "");
                        percentageFixedText.style("fill", d.color);
                        _this.onResize();
                        event.stopPropagation();
                    });
                    this.renderTooltip(path);
                    path.exit().remove();
                    this.onResize();
                };
                // Get all parents of the node
                Sunburst.getTreePath = function (node) {
                    var path = [];
                    var current = node;
                    while (current.parent) {
                        path.unshift(current);
                        current = current.parent;
                    }
                    return path;
                };
                Sunburst.prototype.onResize = function () {
                    var width = this.viewport.width;
                    var height = this.viewport.height;
                    var percentageFixedText = this.svg.select(".sunBurstPercentageFixed");
                    var textWidth = powerbi.TextMeasurementService.measureSvgTextElementWidth(percentageFixedText[0][0]);
                    var textHeight = powerbi.TextMeasurementService.measureSvgTextHeight(percentageFixedText[0][0]);
                    percentageFixedText.style("opacity", 1);
                    percentageFixedText.attr("y", (height / 2 + 4));
                    percentageFixedText.attr("x", ((width / 2) - (textWidth / 2)));
                };
                Sunburst.prototype.highlightPath = function (d, sunBurst, setUnhide) {
                    var parentsArray = d ? Sunburst.getTreePath(d) : [];
                    // Set opacity for all the segments.
                    sunBurst.svg.selectAll("path").each(function () {
                        if (d3.select(this).attr('setUnHide') !== 'true') {
                            d3.select(this).style("opacity", Sunburst.minOpacity);
                        }
                    });
                    // Highlight only ancestors of the current segment.
                    sunBurst.svg.selectAll("path").filter(function (node) {
                        return (parentsArray.indexOf(node) >= 0);
                    }).each(function () {
                        d3.select(this).style("opacity", 1);
                        if (setUnhide === true) {
                            d3.select(this).attr('setUnHide', 'true');
                        }
                    });
                };
                Sunburst.prototype.renderTooltip = function (selection) {
                    visuals.TooltipManager.addTooltip(selection, function (tooltipEvent) {
                        return tooltipEvent.data.tooltipInfo;
                    });
                };
                Sunburst.getTooltipData = function (displayName, value) {
                    return [{
                        displayName: displayName,
                        value: value < 0 ? "" : value.toString()
                    }];
                };
                Sunburst.prototype.covertTreeNodeToSunBurstNode = function (originParentNode, sunburstParentNode, colors, pathIdentity, color) {
                    var selector;
                    if (originParentNode.identity) {
                        pathIdentity = pathIdentity.concat([originParentNode.identity]);
                        selector = { data: pathIdentity, };
                    }
                    var selectionId = pathIdentity.length === 0 ? null : new visuals.SelectionId(selector, false);
                    var valueToSet = originParentNode.values ? originParentNode.values[0].value : 0;
                    var newSunNode = {
                        name: originParentNode.name,
                        value: Math.max(valueToSet, 0),
                        selector: selectionId,
                        key: selectionId ? selectionId.getKey() : null,
                        total: valueToSet
                    };
                    if (originParentNode.value) {
                        newSunNode.color = color ? color : colors.getColor(originParentNode.value).value;
                    }
                    this.total += newSunNode.value;
                    if (originParentNode.children && originParentNode.children.length > 0) {
                        newSunNode.tooltipInfo = Sunburst.getTooltipData(originParentNode.value, -1);
                        newSunNode.children = [];
                        for (var i = 0; i < originParentNode.children.length; i++) {
                            var newChild = this.covertTreeNodeToSunBurstNode(originParentNode.children[i], newSunNode, colors, pathIdentity, newSunNode.color);
                            newSunNode.children.push(newChild);
                            newSunNode.total += newChild.total;
                        }
                    }
                    else {
                        newSunNode.tooltipInfo = Sunburst.getTooltipData(originParentNode.value, valueToSet);
                    }
                    if (sunburstParentNode) {
                        newSunNode.parent = sunburstParentNode;
                    }
                    return newSunNode;
                };
                Sunburst.prototype.converter = function (dataView, colors) {
                    var colorScale = colors.getNewColorScale();
                    this.total = 0;
                    var root = this.covertTreeNodeToSunBurstNode(dataView.matrix.rows.root, null, colorScale, [], undefined);
                    return root;
                };
                Sunburst.minOpacity = 0.2;
                Sunburst.roleNames = {
                    nodes: 'Nodes',
                    values: 'Values',
                };
                Sunburst.capabilities = {
                    dataRoles: [
                        {
                            name: Sunburst.roleNames.nodes,
                            kind: powerbi.VisualDataRoleKind.Grouping,
                            displayName: 'Groups'
                        },
                        {
                            name: Sunburst.roleNames.values,
                            kind: powerbi.VisualDataRoleKind.Measure,
                            displayName: 'Values'
                        }
                    ],
                    objects: {
                        general: {
                            displayName: powerbi.data.createDisplayNameGetter('Visual_General'),
                            properties: {
                                formatString: {
                                    type: { formatting: { formatString: true } },
                                },
                            },
                        },
                    },
                    dataViewMappings: [{
                        conditions: [
                            { 'Groups': { min: 0 }, 'Values': { max: 1 } },
                        ],
                        matrix: {
                            rows: {
                                for: { in: Sunburst.roleNames.nodes },
                            },
                            values: {
                                for: { in: Sunburst.roleNames.values }
                            },
                        }
                    }],
                };
                return Sunburst;
            })();
            Sunburst1445472000808.Sunburst = Sunburst;
        })(Sunburst1445472000808 = visuals.Sunburst1445472000808 || (visuals.Sunburst1445472000808 = {}));
    })(visuals = powerbi.visuals || (powerbi.visuals = {}));
})(powerbi || (powerbi = {}));
var powerbi;
(function (powerbi) {
    var visuals;
    (function (visuals) {
        var plugins;
        (function (plugins) {
            plugins.Sunburst1445472000808 = {
                name: 'Sunburst1445472000808',
                class: 'Sunburst1445472000808',
                capabilities: powerbi.visuals.Sunburst1445472000808.Sunburst.capabilities,
                custom: true,
                create: function () { return new powerbi.visuals.Sunburst1445472000808.Sunburst(); }
            };
        })(plugins = visuals.plugins || (visuals.plugins = {}));
    })(visuals = powerbi.visuals || (powerbi.visuals = {}));
})(powerbi || (powerbi = {}));
//# sourceMappingURL=Sunburst1445472000808.js.map