/*
 *  Power BI Visualizations
 *
 *  Copyright (c) Microsoft Corporation
 *  All rights reserved.
 *  MIT License
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the ""Software""), to deal
 *  in the Software without restriction, including without limitation the rights
 *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED *AS IS*, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 *  THE SOFTWARE.
 */
var powerbi;
(function (powerbi) {
    var visuals;
    (function (visuals) {
        var WaffleChart1453776852267;
        (function (WaffleChart1453776852267) {
            var SelectionManager = visuals.utility.SelectionManager;
            var DataRoleHelper = powerbi.data.DataRoleHelper;
            // DevTools do not support yet `export const waffleChartRoleNames`.
            var waffleChartRoleNames = {
                category: 'Category',
                values: 'Values',
                minValue: 'MinValue',
                maxValue: 'MaxValue',
                paths: 'Paths'
            };
            // DevTools do not support yet `export const waffleChartCapabilities: VisualCapabilities`.
            var waffleChartCapabilities = {
                dataRoles: [{
                    name: waffleChartRoleNames.category,
                    kind: powerbi.VisualDataRoleKind.Grouping,
                    displayName: powerbi.data.createDisplayNameGetter('Role_DisplayName_Group'),
                    description: powerbi.data.createDisplayNameGetter('Role_DisplayName_GroupFunnelDescription')
                }, {
                    name: waffleChartRoleNames.paths,
                    kind: powerbi.VisualDataRoleKind.Grouping,
                    displayName: 'Paths',
                    description: 'The value used to customize the shape of the data points',
                    requiredTypes: [{ text: true }],
                }, {
                    name: waffleChartRoleNames.values,
                    kind: powerbi.VisualDataRoleKind.Measure,
                    displayName: powerbi.data.createDisplayNameGetter('Role_DisplayName_Values'),
                    description: powerbi.data.createDisplayNameGetter('Role_DisplayName_ValuesDescription'),
                    requiredTypes: [{ numeric: true }, { integer: true }],
                }, {
                    name: waffleChartRoleNames.minValue,
                    kind: powerbi.VisualDataRoleKind.Measure,
                    displayName: powerbi.data.createDisplayNameGetter('Role_DisplayName_MinValue'),
                    description: powerbi.data.createDisplayNameGetter('Role_DisplayName_MinValueDescription'),
                    requiredTypes: [{ numeric: true }, { integer: true }],
                }, {
                    name: waffleChartRoleNames.maxValue,
                    kind: powerbi.VisualDataRoleKind.Measure,
                    displayName: powerbi.data.createDisplayNameGetter('Role_DisplayName_MaxValue'),
                    description: powerbi.data.createDisplayNameGetter('Role_DisplayName_MaxValueDescription'),
                    requiredTypes: [{ numeric: true }, { integer: true }],
                }],
                dataViewMappings: [{
                    categorical: {
                        categories: {
                            for: { in: waffleChartRoleNames.category },
                            dataReductionAlgorithm: { top: {} }
                        },
                        values: {
                            group: {
                                by: waffleChartRoleNames.paths,
                                select: [
                                    { bind: { to: waffleChartRoleNames.values } },
                                    { bind: { to: waffleChartRoleNames.minValue } },
                                    { bind: { to: waffleChartRoleNames.maxValue } },
                                ],
                                dataReductionAlgorithm: { top: {} }
                            }
                        },
                        // Specifies a constraint on the number of data rows supported by the visual.
                        rowCount: { preferred: { min: 2 }, supported: { min: 0 } }
                    },
                }],
                objects: {
                    general: {
                        displayName: powerbi.data.createDisplayNameGetter('Visual_General'),
                        properties: {
                            formatString: {
                                type: { formatting: { formatString: true } },
                            },
                        },
                    },
                    dataPoint: {
                        displayName: powerbi.data.createDisplayNameGetter('Visual_DataPoint'),
                        description: powerbi.data.createDisplayNameGetter('Visual_DataPointDescription'),
                        properties: {
                            defaultColor: {
                                displayName: powerbi.data.createDisplayNameGetter('Visual_DefaultColor'),
                                type: { fill: { solid: { color: true } } }
                            },
                            fill: {
                                displayName: powerbi.data.createDisplayNameGetter('Visual_Fill'),
                                type: { fill: { solid: { color: true } } }
                            },
                            fillRule: {
                                displayName: powerbi.data.createDisplayNameGetter('Visual_Gradient'),
                                type: { fillRule: {} },
                                rule: {
                                    inputRole: 'Gradient',
                                    output: {
                                        property: 'fill',
                                        selector: ['Category'],
                                    },
                                },
                            }
                        }
                    }
                },
                sorting: {
                    default: {},
                },
            };
            // DevTools do not support yet `export const waffleChartProps`.
            var waffleChartProps = {
                dataPoint: {
                    defaultColor: { objectName: 'dataPoint', propertyName: 'defaultColor' },
                    fill: { objectName: 'dataPoint', propertyName: 'fill' }
                },
            };
            var WaffleChart = (function () {
                function WaffleChart() {
                }
                WaffleChart.converter = function (dataView) {
                    //console.log(dataView);
                    var labelsArray;
                    var dataType;
                    var identities;
                    var objects;
                    if (dataView.categorical.categories && dataView.categorical.categories.length > 0) {
                        var category0 = dataView.categorical.categories[0];
                        // Copy arrays.
                        labelsArray = category0.values.slice();
                        identities = category0.identity.slice();
                        objects = category0.objects ? category0.objects.slice() : null;
                        dataType = category0.source.type;
                    }
                    if (dataType && dataType.dateTime) {
                        var formatter;
                        for (var i = 0; i < labelsArray.length; i++) {
                            formatter = visuals.valueFormatter.create({
                                format: 'O',
                                value: labelsArray[i],
                                value2: labelsArray[i],
                                tickCount: 6
                            });
                            labelsArray[i] = formatter.format(labelsArray[i]);
                        }
                    }
                    var paths;
                    var totals;
                    var minTotals;
                    var maxTotals;
                    var firstDisplayName;
                    if (dataView.categorical.values && dataView.metadata && dataView.metadata.columns) {
                        var categoricalValues = dataView.categorical.values;
                        var metadataColumns = dataView.metadata.columns;
                        var grouped = categoricalValues.grouped();
                        var pathsGroups = [];
                        for (var i = 0; i < grouped.length; i++) {
                            var pathD = WaffleChart.sanitizePathD(grouped[i].name);
                            pathsGroups.push(pathD);
                        }
                        // This is necessary for backward compatability with Power BI Desktop client Dec 2015
                        // and Jan 2016. 
                        if (DataRoleHelper === undefined) {
                            DataRoleHelper = powerbi.visuals.DataRoleHelper;
                        }
                        // Get the first user display name in case there are no categories selected.
                        if (categoricalValues[0] && categoricalValues[0].source) {
                            firstDisplayName = categoricalValues[0].source.displayName;
                        }
                        // Arrays are got by reference, so, modifying the array modifies the source too.
                        var currentPathIndex = 0;
                        for (var i = 0; i < categoricalValues.length; i++) {
                            var localValues = categoricalValues[i].values;
                            var col = metadataColumns[i];
                            // Note: 'roles' is the wrong way to do this, I don't know why, but use 'source' instead.
                            if (DataRoleHelper.hasRole(categoricalValues[i].source, waffleChartRoleNames.values)) {
                                if (!totals) {
                                    totals = new Array(localValues.length);
                                    for (var j = 0; j < totals.length; j++) {
                                        totals[j] = 0;
                                    }
                                    paths = new Array(localValues.length);
                                }
                                WaffleChart.sumValues(totals, localValues, paths, pathsGroups[currentPathIndex]);
                                // These values were for the current path, move to the next one.
                                currentPathIndex++;
                            }
                            else if (DataRoleHelper.hasRole(categoricalValues[i].source, waffleChartRoleNames.minValue)) {
                                if (!minTotals) {
                                    minTotals = new Array(localValues.length);
                                    for (var j = 0; j < maxTotals.length; j++) {
                                        minTotals[j] = 0;
                                    }
                                }
                                for (var j = 0; j < localValues.length; j++) {
                                    if (localValues[j] !== null) {
                                        minTotals[j] = localValues[j];
                                    }
                                }
                            }
                            else if (DataRoleHelper.hasRole(categoricalValues[i].source, waffleChartRoleNames.maxValue)) {
                                if (!maxTotals) {
                                    maxTotals = new Array(localValues.length);
                                    for (var j = 0; j < maxTotals.length; j++) {
                                        // Picking an arbitrary value different of 0, to minimize the chance of division by zero.
                                        maxTotals[j] = 1;
                                    }
                                }
                                for (var j = 0; j < localValues.length; j++) {
                                    if (localValues[j] !== null) {
                                        maxTotals[j] = localValues[j];
                                    }
                                }
                            }
                        }
                    }
                    var count;
                    if (labelsArray && totals) {
                        count = Math.max(labelsArray.length, totals.length);
                    }
                    else if (labelsArray) {
                        count = labelsArray.length;
                    }
                    else if (totals) {
                        count = totals.length;
                        // When there are no categories set (there should be only one value), set the label to 
                        // the first 'display name'.
                        if (totals.length == 1) {
                            labelsArray = [];
                            labelsArray.push(firstDisplayName);
                        }
                    }
                    else {
                        console.log('No categories or values.');
                    }
                    // If there are no values so far, create an array full of zeros.
                    if (totals === undefined) {
                        totals = [];
                        for (var i = 0; i < count; i++) {
                            totals.push(0);
                        }
                    }
                    if (totals) {
                        // Normalize values.
                        var maxValue = Math.max.apply(null, totals);
                        // If there are max values (and optionally min values), calculate percentage.
                        // If numbers are bellow 100, consider totals are already percentages.
                        // Otherwise, calculate percentages considering the max value in totals to be the 100%.
                        // TODO: Unit test these calculations.
                        if (maxTotals) {
                            for (var i = 0; i < maxTotals.length; i++) {
                                var localMaxValue = maxTotals[i] ? maxTotals[i] : 100;
                                var localMinValue = minTotals && minTotals[i] ? minTotals[i] : 0;
                                var range = localMaxValue - localMinValue;
                                // TODO: Validate that totals[i] is greater than minValues[i].
                                totals[i] = Math.round((totals[i] - localMinValue) * 100 / range);
                            }
                        }
                        else if (maxValue > 100) {
                            // Do cross multiplication.
                            console.log("maxValue: " + maxValue);
                            for (var i = 0; i < totals.length; i++) {
                                totals[i] = Math.round(totals[i] * 100 / maxValue);
                            }
                        }
                    }
                    var viewModel = {
                        labelsArray: labelsArray,
                        identities: identities,
                        objects: objects,
                        values: totals,
                        paths: paths,
                        count: count,
                    };
                    return viewModel;
                };
                WaffleChart.prototype.init = function (options) {
                    this.defaultDataPointColor = 'Coral';
                    this.selectionManager = new SelectionManager({ hostServices: options.host });
                    this.root = d3.select(options.element.get(0)).append('svg');
                    this.debug = new DebugChart();
                    this.debug.init(options);
                };
                WaffleChart.prototype.initWaffles = function (newCount, paths) {
                    // TODO: To improve performance, skip this method if count and paths have not changed.
                    // We need to remove all waffles before creating new ones.
                    if (this.singleWaffleChartArray) {
                        for (var i = 0; i < this.count; i++) {
                            this.singleWaffleChartArray[i].remove();
                        }
                    }
                    this.count = newCount;
                    // Create new waffles.
                    this.singleWaffleChartArray = new Array();
                    for (var i = 0; i < this.count; i++) {
                        this.singleWaffleChartArray[i] = new SingleWaffleChart();
                        this.singleWaffleChartArray[i].init({
                            root: this.root,
                            path: paths ? paths[i] : null,
                        });
                    }
                };
                WaffleChart.prototype.initSelectionManager = function (identities) {
                    var selection = this.root.selectAll('g.singleWaffle');
                    if (!identities) {
                        return;
                    }
                    // Bound data with join-by-index.
                    //console.log("identities");
                    //console.log(identities);
                    selection.data(identities);
                    // Using a local variable to avoid errors later when using 'this'.
                    var selectionManager = this.selectionManager;
                    this.root.on('click', function () {
                        selectionManager.clear().then(function () { return selection.style('opacity', 1); });
                    });
                    selection.on('click', function (d) {
                        var _this = this;
                        //console.log("select():");
                        //console.log(SelectionId.createWithId(d));
                        selectionManager.select(visuals.SelectionId.createWithId(d)).then(function (ids) {
                            if (ids.length > 0) {
                                selection.style('opacity', 0.5);
                                d3.select(_this).style('opacity', 1);
                            }
                            else {
                                selection.style('opacity', 1);
                            }
                        });
                        d3.event.stopPropagation();
                    });
                };
                WaffleChart.prototype.update = function (options) {
                    var debugMessage = undefined;
                    try {
                        if (!options.dataViews || !options.dataViews[0])
                            return;
                        var dataView = this.dataView = options.dataViews[0];
                        debugMessage = JSON.stringify(dataView.categorical.categories);
                        var viewport = options.viewport;
                        var viewModel = WaffleChart.converter(dataView);
                        this.initWaffles(viewModel.count, viewModel.paths);
                        this.initSelectionManager(viewModel.identities);
                    }
                    catch (err) {
                        console.log(err);
                    }
                    //this.debug.update(options, debugMessage);
                    if (dataView.metadata && dataView.metadata.objects) {
                        var defaultColor = powerbi.DataViewObjects.getFillColor(dataView.metadata.objects, waffleChartProps.dataPoint.defaultColor);
                        if (defaultColor) {
                            this.defaultDataPointColor = defaultColor;
                        }
                    }
                    this.root.attr({
                        'height': viewport.height,
                        'width': viewport.width
                    });
                    var bestLayout = WaffleChart.getBestLayout(options.viewport, this.count);
                    var waffleViewport = WaffleChart.getWaffleSize({
                        width: options.viewport.width / bestLayout.columns,
                        height: options.viewport.height / bestLayout.rows,
                    });
                    var globalX = viewport.width / 2 - waffleViewport.width * bestLayout.columns / 2;
                    var globalY = viewport.height / 2 - waffleViewport.height * bestLayout.rows / 2;
                    for (var i = 0; i < this.count; i++) {
                        var localX = globalX + waffleViewport.width * (i % bestLayout.columns);
                        var localY = globalY + waffleViewport.height * Math.floor(i / bestLayout.columns);
                        this.singleWaffleChartArray[i].update({
                            x: localX,
                            y: localY,
                            width: waffleViewport.width,
                            height: waffleViewport.height,
                            fontFamily: 'tahoma',
                            value: viewModel.values && viewModel.values[i] ? viewModel.values[i] : 0,
                            text: viewModel.labelsArray && viewModel.labelsArray[i] ? viewModel.labelsArray[i] : '(Blank)',
                            identity: viewModel.identities && viewModel.identities[i] ? viewModel.identities[i] : null,
                            color: viewModel.objects && viewModel.objects[i] ? WaffleChart.getColor(viewModel.objects[i], this.defaultDataPointColor) : this.defaultDataPointColor,
                        });
                    }
                };
                WaffleChart.getColor = function (objects, defaultDataPointColor) {
                    var colorHelper = new visuals.ColorHelper(new visuals.DataColorPalette(), waffleChartProps.dataPoint.fill, defaultDataPointColor);
                    return colorHelper.getColorForMeasure(objects, "");
                };
                // Return null if path[d] contains invalid characters.
                WaffleChart.sanitizePathD = function (pathD) {
                    var regex = /[bdfgijknopruwxy]+/gi;
                    if (regex.test(pathD)) {
                        return null;
                    }
                    return pathD;
                };
                WaffleChart.sumValues = function (totals, localValues, paths, currentPath) {
                    for (var i = 0; i < localValues.length; i++) {
                        if (localValues[i]) {
                            paths[i] = currentPath;
                            totals[i] += localValues[i];
                        }
                    }
                };
                WaffleChart.getBestLayout = function (viewport, count) {
                    var bestLayout = {
                        rows: 0,
                        columns: 0,
                        totalArea: 0,
                    };
                    for (var i = 1; i <= count; i++) {
                        for (var j = 1; j <= count; j++) {
                            if (i * j >= count) {
                                var currentArea = WaffleChart.calculateArea(viewport, count, i, j);
                                if (currentArea.totalArea > bestLayout.totalArea) {
                                    bestLayout = currentArea;
                                }
                            }
                        }
                    }
                    return bestLayout;
                };
                WaffleChart.calculateArea = function (viewport, count, rows, columns) {
                    var waffleViewport = WaffleChart.getWaffleSize({
                        width: viewport.width / columns,
                        height: viewport.height / rows,
                    });
                    var totalArea = waffleViewport.width * waffleViewport.height * count;
                    return {
                        rows: rows,
                        columns: columns,
                        totalArea: totalArea,
                    };
                };
                WaffleChart.getWaffleSize = function (viewport) {
                    // Cross-multiplications:
                    //     600            : 800
                    //     viewport.width : height
                    //     width          : viewport.height
                    if (viewport.width * 800 / 600 <= viewport.height) {
                        return {
                            width: viewport.width,
                            height: viewport.width * 800 / 600
                        };
                    }
                    return {
                        width: viewport.height * 600 / 800,
                        height: viewport.height
                    };
                };
                WaffleChart.prototype.enumerateObjectInstances = function (options) {
                    var enumeration = new visuals.ObjectEnumerationBuilder();
                    switch (options.objectName) {
                        case 'dataPoint':
                            this.enumerateDataPoints(enumeration);
                            break;
                    }
                    return enumeration.complete();
                };
                WaffleChart.prototype.enumerateDataPoints = function (enumeration) {
                    enumeration.pushInstance({
                        objectName: 'dataPoint',
                        selector: null,
                        properties: {
                            defaultColor: { solid: { color: this.defaultDataPointColor } }
                        },
                    });
                    if (this.singleWaffleChartArray) {
                        for (var i = 0; i < this.singleWaffleChartArray.length; i++) {
                            var singleWaffle = this.singleWaffleChartArray[i];
                            enumeration.pushInstance({
                                objectName: 'dataPoint',
                                displayName: singleWaffle.getText(),
                                selector: visuals.ColorHelper.normalizeSelector(visuals.SelectionId.createWithId(singleWaffle.getIdentity()).getSelector(), false),
                                properties: {
                                    fill: { solid: { color: singleWaffle.getColor() } }
                                },
                            });
                        }
                    }
                };
                WaffleChart.prototype.destroy = function () {
                    console.log("Call to destroy()");
                    this.root = null;
                    for (var i = 0; i < this.count; i++) {
                        this.singleWaffleChartArray[i].destroy();
                    }
                };
                WaffleChart.capabilities = waffleChartCapabilities;
                WaffleChart.DefaultText = 'Invalid DV';
                return WaffleChart;
            })();
            WaffleChart1453776852267.WaffleChart = WaffleChart;
            var SingleWaffleChart = (function () {
                function SingleWaffleChart() {
                }
                SingleWaffleChart.prototype.remove = function () {
                    this.waffleG.remove();
                };
                SingleWaffleChart.prototype.init = function (options) {
                    this.waffleG = options.root.append('g');
                    // Add a class to query this groups by class.
                    this.waffleG.attr('class', 'singleWaffle');
                    this.backgroundRect = this.waffleG.append('rect');
                    this.dotG = this.waffleG.append('g');
                    this.dotBackgroundRect = this.dotG.append('rect');
                    this.dotArray = new Array();
                    for (var i = 0; i < 10; i++) {
                        this.dotArray[i] = new Array(10);
                        for (var j = 0; j < 10; j++) {
                            if (options.path) {
                                this.dotArray[i][j] = this.dotG.append('path').attr('d', options.path);
                            }
                            else {
                                this.dotArray[i][j] = this.dotG.append('circle');
                            }
                        }
                    }
                    // dotArray[i][j][0][0] is a SVGPathElement.
                    this.dotRect = this.dotArray[0][0][0][0].getBBox();
                    this.textG = this.waffleG.append('g');
                    this.percentageText = this.textG.append('text').style('cursor', 'pointer').style('stroke', 'Red').style('stroke-width', '0px').style('font-weight', 'bold').attr('text-anchor', 'middle');
                    this.descText = this.textG.append('text').style('cursor', 'pointer').style('stroke', 'Red').style('stroke-width', '0px').attr('text-anchor', 'middle');
                };
                SingleWaffleChart.prototype.update = function (options) {
                    this.options = options;
                    this.waffleG.attr('transform', 'matrix(1 0 0 1 ' + options.x + ' ' + options.y + ')');
                    this.backgroundRect.style({
                        //'fill': 'LightCyan',
                        fill: 'transparent',
                        'width': options.width,
                        'height': options.height,
                    });
                    var dotGMargin = 20;
                    var dotGX = dotGMargin;
                    var dotGY = dotGMargin;
                    this.dotG.classed('foo', true).attr("transform", function (d) {
                        return "matrix(1 0 0 1 " + dotGX + " " + dotGY + ")";
                    });
                    var dotGHeight = Math.min(options.height, options.width);
                    var chartSide = 0;
                    if (dotGHeight >= dotGMargin * 2) {
                        chartSide = dotGHeight - dotGMargin * 2;
                    }
                    this.dotBackgroundRect.style({
                        //'fill': 'LightCyan',
                        fill: 'transparent',
                        'width': chartSide,
                        'height': chartSide,
                    });
                    var percentage = options.value;
                    var percentageString = percentage.toString() + '%';
                    var dataPointSide = chartSide / 10;
                    var radio = null;
                    var scaleValue = null;
                    var padding = 3;
                    if (this.dotRect && this.dotRect.height > 0 && this.dotRect.width > 0) {
                        // Data point is a path.
                        // Calculate scale using the largest of height or width to make sure the path will stay within
                        // the data point bounds.
                        scaleValue = 0;
                        if (dataPointSide >= padding) {
                            scaleValue = (dataPointSide - padding) / Math.max(this.dotRect.height, this.dotRect.width);
                        }
                    }
                    else {
                        // Data point is a circle.
                        radio = dataPointSide / 2;
                    }
                    for (var i = 9; i >= 0; i--) {
                        for (var j = 0; j < 10; j++) {
                            this.dotArray[i][j].style({
                                'fill': percentage-- > 0 ? this.options.color : 'LightBlue',
                            });
                            if (radio !== null) {
                                this.dotArray[i][j].attr({
                                    'r': radio,
                                    'cy': radio + dataPointSide * i,
                                    'cx': radio + dataPointSide * j,
                                });
                            }
                            else {
                                this.dotArray[i][j].attr({
                                    'transform': 'translate(' + (dataPointSide * j) + ' ' + (dataPointSide * i) + ') scale(' + scaleValue + ')',
                                });
                            }
                        }
                    }
                    var textGX = 0;
                    var textGY = Math.min(options.height, options.width);
                    var textGHeight = Math.max(options.height, options.width) - textGY;
                    this.textG.attr("transform", "matrix(1 0 0 1 " + textGX + " " + textGY + ")");
                    // Divide by 2, because there are 2 lines of code.
                    // And divide by 3 because the 15% on the top and 15% on the bottom should be margin.
                    var percentageTextMargin = textGHeight / 2 / 6;
                    var fontSize = textGHeight / 2 - percentageTextMargin * 2;
                    var percentageTextY = fontSize / 2 + percentageTextMargin;
                    this.percentageText.style({
                        'fill': this.options.color,
                        'font-size': fontSize + 'px',
                        'font-family': options.fontFamily,
                    }).text(percentageString).attr({
                        'y': percentageTextY + 'px',
                        'x': options.width / 2,
                    });
                    var descTextY = textGHeight / 2 + fontSize / 2 + percentageTextMargin;
                    this.descText.style({
                        'fill': 'DarkSlateGray',
                        'font-size': fontSize + 'px',
                        'font-family': options.fontFamily,
                    }).text(this.options.text).attr({
                        'y': descTextY + 'px',
                        'x': options.width / 2,
                    });
                };
                SingleWaffleChart.prototype.destroy = function () {
                };
                SingleWaffleChart.prototype.getText = function () {
                    return this.options.text;
                };
                SingleWaffleChart.prototype.getIdentity = function () {
                    return this.options.identity;
                };
                SingleWaffleChart.prototype.getColor = function () {
                    return this.options.color;
                };
                return SingleWaffleChart;
            })();
            WaffleChart1453776852267.SingleWaffleChart = SingleWaffleChart;
            var DebugChart = (function () {
                function DebugChart() {
                }
                DebugChart.prototype.init = function (options) {
                    this.root = d3.select(options.element.get(0)).select('svg');
                };
                DebugChart.prototype.update = function (options, message) {
                    var viewport = options.viewport;
                    var dataView = options.dataViews[0];
                    var categoricalValues = dataView.categorical.values;
                    if (this.fo) {
                        this.fo.remove();
                    }
                    this.fo = this.root.append('foreignObject');
                    this.fo.attr({
                        'x': 0,
                        'y': 0,
                        'height': viewport.height,
                        'width': viewport.width
                    });
                    var foBody = this.fo.append('xhtml:body');
                    var p = foBody.append('p');
                    p.attr('style', 'font-size:11px;');
                    if (message) {
                        var text = p.text(message);
                    }
                    else {
                        var text = p.text(JSON.stringify(dataView.categorical));
                    }
                };
                return DebugChart;
            })();
            WaffleChart1453776852267.DebugChart = DebugChart;
        })(WaffleChart1453776852267 = visuals.WaffleChart1453776852267 || (visuals.WaffleChart1453776852267 = {}));
    })(visuals = powerbi.visuals || (powerbi.visuals = {}));
})(powerbi || (powerbi = {}));
var powerbi;
(function (powerbi) {
    var visuals;
    (function (visuals) {
        var plugins;
        (function (plugins) {
            plugins.WaffleChart1453776852267 = {
                name: 'WaffleChart1453776852267',
                class: 'WaffleChart1453776852267',
                capabilities: powerbi.visuals.WaffleChart1453776852267.WaffleChart.capabilities,
                custom: true,
                create: function () { return new powerbi.visuals.WaffleChart1453776852267.WaffleChart(); }
            };
        })(plugins = visuals.plugins || (visuals.plugins = {}));
    })(visuals = powerbi.visuals || (powerbi.visuals = {}));
})(powerbi || (powerbi = {}));
