/*
Power BI Visualizations

Copyright (c) Microsoft Corporation

All rights reserved. 

MIT License

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
*/
//// <reference path="../../_references.ts"/>
var powerbi;
(function (powerbi) {
    var visuals;
    (function (visuals) {
        var WordCloud1447959067750;
        (function (WordCloud1447959067750) {
            var ValueFormatter = powerbi.visuals.valueFormatter;
            var getAnimationDuration = visuals.AnimatorCommon.GetAnimationDuration;
            (function (WordCloudScaleType) {
                WordCloudScaleType[WordCloudScaleType["logn"] = 0] = "logn";
                WordCloudScaleType[WordCloudScaleType["sqrt"] = 1] = "sqrt";
                WordCloudScaleType[WordCloudScaleType["value"] = 2] = "value";
            })(WordCloud1447959067750.WordCloudScaleType || (WordCloud1447959067750.WordCloudScaleType = {}));
            var WordCloudScaleType = WordCloud1447959067750.WordCloudScaleType;
            ;
            var VisualLayout = (function () {
                function VisualLayout(defaultViewport, defaultMargin) {
                    this.defaultViewport = defaultViewport || { width: 0, height: 0 };
                    this.defaultMargin = defaultMargin || { top: 0, bottom: 0, right: 0, left: 0 };
                }
                Object.defineProperty(VisualLayout.prototype, "margin", {
                    get: function () {
                        return this.marginValue || (this.margin = this.defaultMargin);
                    },
                    set: function (value) {
                        this.marginValue = VisualLayout.restrictToMinMax(value);
                        this.update();
                    },
                    enumerable: true,
                    configurable: true
                });
                Object.defineProperty(VisualLayout.prototype, "viewport", {
                    get: function () {
                        return this.viewportValue || (this.viewportValue = this.defaultViewport);
                    },
                    set: function (value) {
                        this.viewportValue = VisualLayout.restrictToMinMax(value);
                        this.update();
                    },
                    enumerable: true,
                    configurable: true
                });
                Object.defineProperty(VisualLayout.prototype, "viewportIn", {
                    get: function () {
                        return this.viewportInValue || this.viewport;
                    },
                    enumerable: true,
                    configurable: true
                });
                Object.defineProperty(VisualLayout.prototype, "viewportInIsZero", {
                    get: function () {
                        return this.viewportIn.width === 0 || this.viewportIn.height === 0;
                    },
                    enumerable: true,
                    configurable: true
                });
                VisualLayout.prototype.update = function () {
                    this.viewportInValue = VisualLayout.restrictToMinMax({
                        width: this.viewport.width - (this.margin.left + this.margin.right),
                        height: this.viewport.height - (this.margin.top + this.margin.bottom)
                    });
                };
                VisualLayout.restrictToMinMax = function (value) {
                    var result = $.extend({}, value);
                    d3.keys(value).forEach(function (x) { return result[x] = Math.max(0, value[x]); });
                    return result;
                };
                return VisualLayout;
            })();
            var WordCloud = (function () {
                function WordCloud(options) {
                    this.durationAnimations = 500;
                    this.fakeViewport = {
                        width: 1500,
                        height: 1000
                    };
                    this.canvasViewport = {
                        width: 128,
                        height: 2048
                    };
                    if (options) {
                        this.svg = options.svg || this.svg;
                        this.layout = new VisualLayout(null, options.margin || WordCloud.DefaultMargin);
                        if (options.animator)
                            this.animator = options.animator;
                    }
                }
                WordCloud.prototype.init = function (options) {
                    var _this = this;
                    if (this.svg)
                        this.root = this.svg;
                    else
                        this.root = d3.select(options.element.get(0)).append("svg");
                    WordCloud.colors = options.style.colorPalette.dataColors;
                    this.hostService = options.host;
                    this.selectionManager = new visuals.utility.SelectionManager({ hostServices: this.hostService });
                    if (!this.layout)
                        this.layout = new VisualLayout(null, WordCloud.DefaultMargin);
                    this.root.classed(WordCloud.ClassName, true);
                    this.root.on("click", function () {
                        _this.selectionManager.clear();
                        _this.setSelection(_this.wordsSelection);
                    });
                    this.fontFamily = this.root.style("font-family");
                    this.main = this.root.append("g");
                    this.wordsContainerSelection = this.main
                        .append("g")
                        .classed(WordCloud.Words["class"], true);
                    this.canvas = document.createElement("canvas");
                };
                WordCloud.prototype.converter = function (dataView) {
                    var _this = this;
                    if (!dataView ||
                        !dataView.categorical ||
                        !dataView.categorical.categories ||
                        !dataView.categorical.categories[0] ||
                        !dataView.categorical.categories[0].values ||
                        !dataView.categorical.categories[0].values.length ||
                        !(dataView.categorical.categories[0].values.length > 0))
                        return null;
                    var categories = dataView.categorical.categories[0].values, settings = WordCloud.parseSettings(dataView, categories[0]), frequencies, texts;
                    if (!settings)
                        return null;
                    if (!_.isEmpty(dataView.categorical.values) &&
                        !_.isEmpty(dataView.categorical.values[0]) &&
                        !_.isEmpty(dataView.categorical.values[0].values))
                        frequencies = dataView.categorical.values[0].values;
                    texts = categories.map(function (item, index) {
                        var color, categoryObject = dataView.categorical.categories[0];
                        if (categoryObject.objects && categoryObject.objects[index])
                            color = _this.getColor(WordCloud.Properties.dataPoint.fill, explore.util.getRandomColor(), categoryObject.objects[index]);
                        else {
                            if (_this.wordCloudTexts && _this.wordCloudTexts[index])
                                color = _this.wordCloudTexts[index].color;
                            else
                                color = explore.util.getRandomColor();
                        }
                        return {
                            text: item,
                            count: (frequencies && frequencies[index] && !isNaN(frequencies[index])) ? frequencies[index] : 1,
                            index: index,
                            selectionId: visuals.SelectionId.createWithId(dataView.categorical.categories[0].identity[index]),
                            color: color,
                        };
                    });
                    return {
                        settings: settings,
                        texts: texts
                    };
                };
                WordCloud.prototype.getColor = function (properties, defaultColor, objects) {
                    var colorHelper;
                    colorHelper = new visuals.ColorHelper(WordCloud.colors, properties, defaultColor);
                    return explore.util.hexToRgb(colorHelper.getColorForMeasure(objects, ""));
                };
                WordCloud.parseSettings = function (dataView, value) {
                    if (!dataView ||
                        !dataView.metadata ||
                        !dataView.metadata.columns ||
                        !dataView.metadata.columns[0])
                        return null;
                    var objects = dataView.metadata.objects, valueFormatter, minFontSize, maxFontSize, minAngle, maxAngle, maxNumberOfOrientations, isRotateText = false, isBrokenText = true, isRemoveStopWords = true, stopWords, stopWordsArray, isDefaultStopWords = false, maxNumberOfWords;
                    maxNumberOfWords = WordCloud.getNumberFromObjects(objects, WordCloud.Properties.general.maxNumberOfWords, WordCloud.DefaultSettings.maxNumberOfWords);
                    minFontSize = WordCloud.getNumberFromObjects(objects, WordCloud.Properties.general.minFontSize, WordCloud.DefaultSettings.minFontSize);
                    maxFontSize = WordCloud.getNumberFromObjects(objects, WordCloud.Properties.general.maxFontSize, WordCloud.DefaultSettings.maxFontSize);
                    minAngle = WordCloud.getNumberFromObjects(objects, WordCloud.Properties.rotateText.minAngle, WordCloud.DefaultSettings.minAngle);
                    maxAngle = WordCloud.getNumberFromObjects(objects, WordCloud.Properties.rotateText.maxAngle, WordCloud.DefaultSettings.maxAngle);
                    isRotateText = powerbi.DataViewObjects.getValue(objects, WordCloud.Properties.rotateText.show, WordCloud.DefaultSettings.isRotateText);
                    maxNumberOfOrientations = WordCloud.getNumberFromObjects(objects, WordCloud.Properties.rotateText.maxNumberOfOrientations, WordCloud.DefaultSettings.maxNumberOfOrientations);
                    valueFormatter = ValueFormatter.create({
                        format: ValueFormatter.getFormatString(dataView.categorical.categories[0].source, WordCloud.Properties.general.formatString),
                        value: value
                    });
                    isBrokenText = powerbi.DataViewObjects.getValue(objects, WordCloud.Properties.general.isBrokenText, WordCloud.DefaultSettings.isBrokenText);
                    isRemoveStopWords = powerbi.DataViewObjects.getValue(objects, WordCloud.Properties.stopWords.show, WordCloud.DefaultSettings.isRemoveStopWords);
                    stopWords = powerbi.DataViewObjects.getValue(objects, WordCloud.Properties.stopWords.words, WordCloud.DefaultSettings.stopWords);
                    if (typeof stopWords === "string")
                        stopWordsArray = stopWords.split(WordCloud.StopWordsDelemiter);
                    else
                        stopWordsArray = WordCloud.DefaultSettings.stopWordsArray;
                    isDefaultStopWords = powerbi.DataViewObjects.getValue(objects, WordCloud.Properties.stopWords.isDefaultStopWords, WordCloud.DefaultSettings.isDefaultStopWords);
                    return {
                        minFontSize: minFontSize,
                        maxFontSize: maxFontSize,
                        minAngle: minAngle,
                        maxAngle: maxAngle,
                        maxNumberOfOrientations: maxNumberOfOrientations,
                        valueFormatter: valueFormatter,
                        isRotateText: isRotateText,
                        isBrokenText: isBrokenText,
                        isRemoveStopWords: isRemoveStopWords,
                        stopWords: stopWords,
                        stopWordsArray: stopWordsArray,
                        isDefaultStopWords: isDefaultStopWords,
                        maxNumberOfWords: maxNumberOfWords
                    };
                };
                WordCloud.getNumberFromObjects = function (objects, properties, defaultValue) {
                    return objects ? powerbi.DataViewObjects.getValue(objects, properties, defaultValue) : defaultValue;
                };
                WordCloud.prototype.parseNumber = function (value, defaultValue, minValue, maxValue) {
                    if (defaultValue === void 0) { defaultValue = 0; }
                    if (minValue === void 0) { minValue = -Number.MAX_VALUE; }
                    if (maxValue === void 0) { maxValue = Number.MAX_VALUE; }
                    var parsedValue = Number(value);
                    if (isNaN(parsedValue) || (typeof value === "string" && value.length === 0))
                        return defaultValue;
                    if (parsedValue < minValue)
                        return minValue;
                    if (parsedValue > maxValue)
                        return maxValue;
                    return parsedValue;
                };
                WordCloud.prototype.computePositions = function (words, onPositionsComputed) {
                    var _this = this;
                    var context = this.getCanvasContext(), surface = [], borders = null, maxNumberOfWords;
                    if (!words || !(words.length > 0))
                        return null;
                    maxNumberOfWords = Math.abs(this.parseNumber(this.settings.maxNumberOfWords, WordCloud.DefaultSettings.maxNumberOfWords, words.length * -1, words.length));
                    if (words.length > maxNumberOfWords)
                        words = words.slice(0, maxNumberOfWords);
                    for (var i = void 0; i < (this.specialViewport.width >> 5) * this.specialViewport.height; i++) {
                        surface[i] = 0;
                    }
                    setTimeout(function () { return _this.computeCycle(words, context, surface, borders, onPositionsComputed, [], 0); }, 0);
                };
                WordCloud.prototype.computeCycle = function (words, context, surface, borders, onPositionsComputed, wordsForDraw, index) {
                    if (wordsForDraw === void 0) { wordsForDraw = []; }
                    if (index === void 0) { index = 0; }
                    var word = words[index], ratio = 1;
                    if (words.length <= 10)
                        ratio = 5;
                    else if (words.length <= 25)
                        ratio = 3;
                    else if (words.length <= 75)
                        ratio = 1.5;
                    else if (words.length <= 100)
                        ratio = 1.25;
                    word.x = (this.specialViewport.width / ratio * (Math.random() + 0.5)) >> 1;
                    word.y = (this.specialViewport.height / ratio * (Math.random() + 0.5)) >> 1;
                    this.generateSprites(context, word, words, index);
                    if (word.sprite && this.findPosition(surface, word, borders)) {
                        wordsForDraw.push(word);
                        borders = this.updateBorders(word, borders);
                        word.x -= this.specialViewport.width >> 1;
                        word.y -= this.specialViewport.height >> 1;
                    }
                    if (++index < words.length && this.root)
                        this.computeCycle(words, context, surface, borders, onPositionsComputed, wordsForDraw, index);
                    else {
                        onPositionsComputed({
                            data: wordsForDraw,
                            leftBorder: borders && borders[0],
                            rightBorder: borders && borders[1]
                        });
                    }
                };
                WordCloud.prototype.updateBorders = function (word, borders) {
                    if (borders && borders.length === 2) {
                        var leftBorder = borders[0], rightBorder = borders[1];
                        if (word.x + word.x0 < leftBorder.x)
                            leftBorder.x = word.x + word.x0;
                        if (word.y + word.y0 < leftBorder.y)
                            leftBorder.y = word.y + word.y0;
                        if (word.x + word.x1 > rightBorder.x)
                            rightBorder.x = word.x + word.x1;
                        if (word.y + word.y1 > rightBorder.y)
                            rightBorder.y = word.y + word.y1;
                    }
                    else {
                        borders = [
                            {
                                x: word.x + word.x0,
                                y: word.y + word.y0
                            }, {
                                x: word.x + word.x1,
                                y: word.y + word.y1
                            }
                        ];
                    }
                    return borders;
                };
                WordCloud.prototype.generateSprites = function (context, currentWord, words, index) {
                    if (currentWord.sprite)
                        return;
                    context.clearRect(0, 0, this.canvasViewport.width << 5, this.canvasViewport.height);
                    var x = 0, y = 0, maxHeight = 0, quantityOfWords = words.length, pixels, sprite = [];
                    for (var i = index; i < quantityOfWords; i++) {
                        var currentWordData = words[i], widthOfWord = 0, heightOfWord = 0;
                        context.save();
                        context.font = "normal normal " + (currentWordData.size + 1) + WordCloud.Size + " " + this.fontFamily;
                        widthOfWord = context.measureText(currentWordData.text + "m").width;
                        heightOfWord = currentWordData.size << 1;
                        if (currentWordData.rotate) {
                            var sr = Math.sin(currentWordData.rotate * WordCloud.Radians), cr = Math.cos(currentWordData.rotate * WordCloud.Radians), widthCr = widthOfWord * cr, widthSr = widthOfWord * sr, heightCr = heightOfWord * cr, heightSr = heightOfWord * sr;
                            widthOfWord = (Math.max(Math.abs(widthCr + heightSr), Math.abs(widthCr - heightSr)) + 31) >> 5 << 5;
                            heightOfWord = Math.floor(Math.max(Math.abs(widthSr + heightCr), Math.abs(widthSr - heightCr)));
                        }
                        else
                            widthOfWord = (widthOfWord + 31) >> 5 << 5;
                        if (heightOfWord > maxHeight)
                            maxHeight = heightOfWord;
                        if (x + widthOfWord >= (this.canvasViewport.width << 5)) {
                            x = 0;
                            y += maxHeight;
                            maxHeight = 0;
                        }
                        context.translate((x + (widthOfWord >> 1)), (y + (heightOfWord >> 1)));
                        if (currentWordData.rotate)
                            context.rotate(currentWordData.rotate * WordCloud.Radians);
                        context.fillText(currentWordData.text, 0, 0);
                        if (currentWordData.padding) {
                            context.lineWidth = 2 * currentWordData.padding;
                            context.strokeText(currentWordData.text, 0, 0);
                        }
                        context.restore();
                        currentWordData.width = widthOfWord;
                        currentWordData.height = heightOfWord;
                        currentWordData.xOff = x;
                        currentWordData.yOff = y;
                        currentWordData.x1 = widthOfWord >> 1;
                        currentWordData.y1 = heightOfWord >> 1;
                        currentWordData.x0 = -currentWordData.x1;
                        currentWordData.y0 = -currentWordData.y1;
                        x += widthOfWord;
                    }
                    pixels = context.getImageData(0, 0, this.canvasViewport.width << 5, this.canvasViewport.height).data;
                    sprite = [];
                    for (var i = quantityOfWords - 1; i >= 0; i--) {
                        var currentWordData = words[i], width = currentWordData.width, width32 = width >> 5, height = currentWordData.y1 - currentWordData.y0, x_1 = 0, y_1 = 0, seen = 0, seenRow = 0;
                        if (currentWordData.xOff + width >= (this.canvasViewport.width << 5) ||
                            currentWordData.yOff + height >= this.canvasViewport.height) {
                            currentWordData.sprite = null;
                            continue;
                        }
                        for (var j = 0; j < height * width32; j++) {
                            sprite[j] = 0;
                        }
                        if (currentWordData.xOff !== null)
                            x_1 = currentWordData.xOff;
                        else
                            return;
                        y_1 = currentWordData.yOff;
                        seen = 0;
                        seenRow = -1;
                        for (var j = 0; j < height; j++) {
                            for (var k = 0; k < width; k++) {
                                var l = width32 * j + (k >> 5), index_1 = ((y_1 + j) * (this.canvasViewport.width << 5) + (x_1 + k)) << 2, m = pixels[index_1]
                                    ? 1 << (31 - (k % 32))
                                    : 0;
                                sprite[l] |= m;
                                seen |= m;
                            }
                            if (seen)
                                seenRow = j;
                            else {
                                currentWordData.y0++;
                                height--;
                                j--;
                                y_1++;
                            }
                        }
                        currentWordData.y1 = currentWordData.y0 + seenRow;
                        currentWordData.sprite = sprite.slice(0, (currentWordData.y1 - currentWordData.y0) * width32);
                    }
                };
                WordCloud.prototype.findPosition = function (surface, word, borders) {
                    var startPoint = { x: word.x, y: word.y }, delta = Math.sqrt(this.specialViewport.width * this.specialViewport.width + this.specialViewport.height * this.specialViewport.height), point, dt = Math.random() < 0.5 ? 1 : -1, shift = -dt, dx, dy;
                    while (true) {
                        shift += dt;
                        point = this.archimedeanSpiral(shift);
                        dx = Math.floor(point.x);
                        dy = Math.floor(point.y);
                        if (Math.min(Math.abs(dx), Math.abs(dy)) >= delta)
                            break;
                        word.x = startPoint.x + dx;
                        word.y = startPoint.y + dy;
                        if (word.x + word.x0 < 0 ||
                            word.y + word.y0 < 0 ||
                            word.x + word.x1 > this.specialViewport.width ||
                            word.y + word.y1 > this.specialViewport.height)
                            continue;
                        if (!borders || !this.checkIntersect(word, surface)) {
                            if (!borders || this.checkIntersectOfRectangles(word, borders[0], borders[1])) {
                                var sprite = word.sprite, width = word.width >> 5, shiftWidth = this.specialViewport.width >> 5, lx = word.x - (width << 4), sx = lx & 127, msx = 32 - sx, height = word.y1 - word.y0, x = (word.y + word.y0) * shiftWidth + (lx >> 5);
                                for (var i = 0; i < height; i++) {
                                    var lastSprite = 0;
                                    for (var j = 0; j <= width; j++) {
                                        var leftMask = lastSprite << msx, rightMask = void 0;
                                        if (j < width)
                                            lastSprite = sprite[i * width + j];
                                        rightMask = j < width
                                            ? lastSprite >>> sx
                                            : 0;
                                        surface[x + j] |= leftMask | rightMask;
                                    }
                                    x += shiftWidth;
                                }
                                word.sprite = null;
                                return true;
                            }
                        }
                    }
                    return false;
                };
                WordCloud.prototype.archimedeanSpiral = function (value) {
                    var ratio = this.specialViewport.width / this.specialViewport.height;
                    value = value * 0.1;
                    return {
                        x: ratio * value * Math.cos(value),
                        y: value * Math.sin(value)
                    };
                };
                WordCloud.prototype.checkIntersect = function (word, surface) {
                    var shiftWidth = this.specialViewport.width >> 5, sprite = word.sprite, widthOfWord = word.width >> 5, lx = word.x - (widthOfWord << 4), sx = lx & 127, msx = 32 - sx, heightOfWord = word.y1 - word.y0, x = (word.y + word.y0) * shiftWidth + (lx >> 5);
                    for (var i = 0; i < heightOfWord; i++) {
                        var lastSprite = 0;
                        for (var j = 0; j <= widthOfWord; j++) {
                            var mask = 0, leftMask = void 0, intersectMask = 0;
                            leftMask = lastSprite << msx;
                            if (j < widthOfWord)
                                lastSprite = sprite[i * widthOfWord + j];
                            mask = j < widthOfWord
                                ? lastSprite >>> sx
                                : 0;
                            intersectMask = (leftMask | mask) & surface[x + j];
                            if (intersectMask)
                                return true;
                        }
                        x += shiftWidth;
                    }
                    return false;
                };
                WordCloud.prototype.checkIntersectOfRectangles = function (word, leftBorder, rightBorder) {
                    return (word.x + word.x1) > leftBorder.x &&
                        (word.x + word.x0) < rightBorder.x &&
                        (word.y + word.y1) > leftBorder.y &&
                        (word.y + word.y0) < rightBorder.y;
                };
                WordCloud.prototype.getCanvasContext = function () {
                    if (!this.canvasViewport)
                        return null;
                    this.canvas.width = 1;
                    this.canvas.height = 1;
                    var context = this.canvas.getContext("2d");
                    this.canvas.width = this.canvasViewport.width << 5;
                    this.canvas.height = this.canvasViewport.height;
                    context = this.canvas.getContext("2d");
                    context.fillStyle = context.strokeStyle = "red";
                    context.textAlign = "center";
                    return context;
                };
                WordCloud.prototype.getReducedText = function (texts) {
                    var brokenStrings = [];
                    brokenStrings = this.getBrokenWords(texts);
                    return brokenStrings.reduce(function (previousValue, currentValue) {
                        if (!previousValue.some(function (value) {
                            if (value.index !== currentValue.index && value.text === currentValue.text) {
                                value.count += currentValue.count;
                                return true;
                            }
                            return false;
                        })) {
                            previousValue.push(currentValue);
                        }
                        return previousValue;
                    }, []);
                };
                WordCloud.prototype.getBrokenWords = function (words) {
                    var _this = this;
                    var brokenStrings = [], whiteSpaceRegExp = /\s/, punctuatuinRegExp;
                    if (!this.settings.isBrokenText)
                        return words;
                    punctuatuinRegExp = new RegExp("[" + WordCloud.Punctuation.join("\\") + "]", "gim");
                    words.forEach(function (item) {
                        if (typeof item.text === "string") {
                            var words_1;
                            words_1 = item.text.replace(punctuatuinRegExp, " ").split(whiteSpaceRegExp);
                            if (_this.settings.isRemoveStopWords) {
                                var stopWords = _this.settings.stopWordsArray;
                                if (_this.settings.isDefaultStopWords)
                                    stopWords = stopWords.concat(WordCloud.StopWords);
                                words_1 = words_1.filter(function (value) {
                                    return value.length > 0 && !stopWords.some(function (removeWord) {
                                        return value.toLocaleLowerCase() === removeWord.toLocaleLowerCase();
                                    });
                                });
                            }
                            words_1.forEach(function (element) {
                                if (element.length > 0 && !whiteSpaceRegExp.test(element)) {
                                    brokenStrings.push({
                                        text: element,
                                        count: item.count,
                                        index: item.index,
                                        selectionId: item.selectionId,
                                        color: item.color
                                    });
                                }
                            });
                        }
                        else
                            brokenStrings.push(item);
                    });
                    return brokenStrings;
                };
                WordCloud.prototype.getWords = function (values) {
                    var _this = this;
                    var sortedValues, minValue = 0, maxValue = 0, valueFormatter = this.settings.valueFormatter;
                    if (!values || !(values.length >= 1))
                        return [];
                    sortedValues = values.sort(function (a, b) {
                        return b.count - a.count;
                    });
                    minValue = sortedValues[sortedValues.length - 1].count;
                    maxValue = sortedValues[0].count;
                    var returnValues = values.map(function (value) {
                        return {
                            text: valueFormatter.format(value.text),
                            size: _this.getFontSize(value.count, minValue, maxValue),
                            x: 0,
                            y: 0,
                            rotate: _this.getAngle(),
                            padding: 1,
                            width: 0,
                            height: 0,
                            xOff: 0,
                            yOff: 0,
                            x0: 0,
                            y0: 0,
                            x1: 0,
                            y1: 0,
                            color: value.color,
                            selectionId: value.selectionId,
                            wordIndex: value.index
                        };
                    });
                    this.dataBeforeRender = returnValues;
                    return returnValues;
                };
                WordCloud.prototype.getFontSize = function (value, minValue, maxValue, scaleType) {
                    if (scaleType === void 0) { scaleType = WordCloudScaleType.value; }
                    var weight, fontSize, maxFontSize, minFontSize;
                    minFontSize = Math.abs(this.parseNumber(this.settings.minFontSize, WordCloud.DefaultSettings.minFontSize));
                    maxFontSize = Math.abs(this.parseNumber(this.settings.maxFontSize, WordCloud.DefaultSettings.maxFontSize));
                    if (minFontSize > maxFontSize) {
                        var buffer = minFontSize;
                        minFontSize = maxFontSize;
                        maxFontSize = buffer;
                    }
                    switch (scaleType) {
                        case WordCloudScaleType.logn: {
                            weight = Math.log(value);
                        }
                        case WordCloudScaleType.sqrt: {
                            weight = Math.sqrt(value);
                        }
                        case WordCloudScaleType.value: {
                            weight = value;
                        }
                    }
                    fontSize = weight > minValue
                        ? (maxFontSize * (weight - minValue)) / (maxValue - minValue)
                        : 0;
                    fontSize = (fontSize * 100) / maxFontSize;
                    fontSize = (fontSize * (maxFontSize - minFontSize)) / 100 + minFontSize;
                    return fontSize;
                };
                WordCloud.prototype.getAngle = function () {
                    if (!this.settings ||
                        !this.settings.isRotateText)
                        return 0;
                    var minAngle, maxAngle, maxNumberOfOrientations, angle;
                    maxNumberOfOrientations = Math.abs(this.parseNumber(this.settings.maxNumberOfOrientations, 0));
                    minAngle = this.parseNumber(this.settings.minAngle, 0, WordCloud.MinAngle, WordCloud.MaxAngle);
                    maxAngle = this.parseNumber(this.settings.maxAngle, 0, WordCloud.MinAngle, WordCloud.MaxAngle);
                    if (minAngle > maxAngle) {
                        var buffer = minAngle;
                        minAngle = maxAngle;
                        maxAngle = buffer;
                    }
                    angle = Math.abs(((maxAngle - minAngle) / maxNumberOfOrientations) * Math.floor(Math.random() * maxNumberOfOrientations));
                    return maxNumberOfOrientations !== 0 ? minAngle + angle : 0;
                };
                WordCloud.prototype.update = function (visualUpdateOptions) {
                    var _this = this;
                    if (!visualUpdateOptions ||
                        !visualUpdateOptions.viewport ||
                        !visualUpdateOptions.dataViews ||
                        !visualUpdateOptions.dataViews[0] ||
                        !visualUpdateOptions.viewport ||
                        !(visualUpdateOptions.viewport.height >= 0) ||
                        !(visualUpdateOptions.viewport.width >= 0))
                        return;
                    this.visualUpdateOptions = visualUpdateOptions;
                    this.layout.viewport = this.visualUpdateOptions.viewport;
                    var dataView = visualUpdateOptions.dataViews[0];
                    if (this.layout.viewportInIsZero)
                        return;
                    this.durationAnimations = getAnimationDuration(this.animator, visualUpdateOptions.suppressAnimations);
                    this.UpdateSize();
                    this.data = this.converter(dataView);
                    if (!this.data)
                        return;
                    this.settings = this.data.settings;
                    this.wordCloudTexts = this.data.texts;
                    this.computePositions(this.getWords(this.getReducedText(this.data.texts)), function (wordCloudDataView) { return _this.render(wordCloudDataView); });
                    if (visualUpdateOptions !== this.visualUpdateOptions)
                        this.update(this.visualUpdateOptions);
                };
                WordCloud.prototype.UpdateSize = function () {
                    var fakeWidth, fakeHeight, ratio;
                    ratio = Math.sqrt((this.fakeViewport.width * this.fakeViewport.height)
                        / (this.layout.viewportIn.width * this.layout.viewportIn.height));
                    if (isNaN(ratio))
                        fakeHeight = fakeWidth = 1;
                    else {
                        fakeHeight = this.layout.viewportIn.height * ratio;
                        fakeWidth = this.layout.viewportIn.width * ratio;
                    }
                    this.specialViewport = {
                        height: fakeHeight,
                        width: fakeWidth
                    };
                    this.root.attr({
                        "height": this.layout.viewport.height,
                        "width": this.layout.viewport.width
                    });
                };
                WordCloud.prototype.render = function (wordCloudDataView) {
                    var _this = this;
                    if (!wordCloudDataView ||
                        !wordCloudDataView.data)
                        return;
                    this.wordCloudDataView = wordCloudDataView;
                    var animatedWordSelection, wordElements = this.main
                        .select(WordCloud.Words.selector)
                        .selectAll(WordCloud.Word.selector);
                    this.wordsSelection = wordElements.data(wordCloudDataView.data);
                    this.animation(this.wordsSelection, this.durationAnimations)
                        .attr("transform", function (item) { return (visuals.SVGUtil.translate(item.x, item.y) + "rotate(" + item.rotate + ")"); })
                        .style({
                        "font-size": (function (item) { return ("" + item.size + WordCloud.Size); }),
                        "fill": (function (item) { return item.color; }),
                    });
                    animatedWordSelection = this.wordsSelection
                        .enter()
                        .append("svg:text")
                        .attr("transform", function (item) { return (visuals.SVGUtil.translate(item.x, item.y) + "rotate(" + item.rotate + ")"); })
                        .style("font-size", "1px");
                    this.wordsSelection.on("click", function (item) {
                        _this.selectionManager
                            .select(item.selectionId, d3.event.ctrlKey)
                            .then(function () { return _this.setSelection(_this.wordsSelection); });
                        d3.event.stopPropagation();
                    });
                    this.animation(animatedWordSelection, this.durationAnimations)
                        .style({
                        "font-size": (function (item) { return ("" + item.size + WordCloud.Size); }),
                        "fill": (function (item) { return item.color; }),
                    });
                    this.wordsSelection
                        .text(function (item) { return item.text; })
                        .classed(WordCloud.Word["class"], true);
                    this.wordsSelection.exit().remove();
                    this.setSelection(this.wordsSelection);
                    setTimeout(function () {
                        if (_this.root)
                            _this.scaleMainView(wordCloudDataView, wordElements[0].length && _this.durationAnimations);
                    }, this.durationAnimations + WordCloud.RenderDelay);
                };
                WordCloud.prototype.setSelection = function (selection) {
                    var selectionIds = this.selectionManager.getSelectionIds();
                    if (selectionIds.some(function (x) { return !selection.data().some(function (d) { return d.selectionId.getKey() === x.getKey(); }); })) {
                        this.selectionManager.clear();
                        selectionIds = [];
                    }
                    if (!selectionIds.length) {
                        this.setOpacity(selection, WordCloud.MaxOpacity, true);
                        return;
                    }
                    var selectedColumns = selection.filter(function (x) {
                        return selectionIds.some(function (y) { return y.getKey() === x.selectionId.getKey(); });
                    });
                    this.setOpacity(selection, WordCloud.MinOpacity);
                    this.setOpacity(selectedColumns, WordCloud.MaxOpacity);
                };
                WordCloud.prototype.setOpacity = function (element, opacityValue, disableAnimation) {
                    if (disableAnimation === void 0) { disableAnimation = false; }
                    var elementAnimation = disableAnimation ? element : this.animation(element);
                    elementAnimation.style("fill-opacity", opacityValue);
                };
                WordCloud.prototype.scaleMainView = function (wordCloudDataView, durationAnimation) {
                    if (durationAnimation === void 0) { durationAnimation = 0; }
                    if (!wordCloudDataView ||
                        !wordCloudDataView.leftBorder ||
                        !wordCloudDataView.rightBorder)
                        return;
                    var scale = 1, mainSVGRect = this.main.node()["getBBox"](), leftBorder = wordCloudDataView.leftBorder, rightBorder = wordCloudDataView.rightBorder, width2, height2, scaleByX, scaleByY;
                    scaleByX = this.layout.viewportIn.width / Math.abs(leftBorder.x - rightBorder.x);
                    scaleByY = this.layout.viewportIn.height / Math.abs(leftBorder.y - rightBorder.y);
                    scale = Math.min(scaleByX, scaleByY);
                    width2 = this.layout.margin.left + (mainSVGRect.x * scale * -1)
                        + (this.layout.viewportIn.width - (mainSVGRect.width * scale)) / 2;
                    height2 = this.layout.margin.top + (mainSVGRect.y * scale * -1)
                        + (this.layout.viewportIn.height - (mainSVGRect.height * scale)) / 2;
                    this.animation(this.main, durationAnimation)
                        .attr("transform", visuals.SVGUtil.translate(width2, height2) + "scale(" + scale + ")");
                };
                WordCloud.prototype.enumerateObjectInstances = function (options) {
                    var _this = this;
                    var instances = [];
                    if (!this.settings)
                        return instances;
                    switch (options.objectName) {
                        case "general": {
                            var general = {
                                objectName: "general",
                                displayName: "general",
                                selector: null,
                                properties: {
                                    maxNumberOfWords: this.settings.maxNumberOfWords,
                                    minFontSize: this.settings.minFontSize,
                                    maxFontSize: this.settings.maxFontSize,
                                    isBrokenText: this.settings.isBrokenText
                                }
                            };
                            instances.push(general);
                            break;
                        }
                        case "dataPoint": {
                            if (!this.wordCloudDataView ||
                                !this.wordCloudDataView.data)
                                return;
                            var dataPoints = this.dataBeforeRender;
                            var wordCategoriesIndex = [];
                            dataPoints.forEach(function (item) {
                                if (wordCategoriesIndex.indexOf(item.wordIndex) === -1) {
                                    wordCategoriesIndex.push(item.wordIndex);
                                    instances.push({
                                        objectName: "dataPoint",
                                        displayName: _this.data.texts[item.wordIndex].text,
                                        selector: visuals.ColorHelper.normalizeSelector(item.selectionId.getSelector(), false),
                                        properties: {
                                            fill: { solid: { color: item.color } }
                                        }
                                    });
                                }
                            });
                            break;
                        }
                        case "rotateText": {
                            var rotateText = {
                                objectName: "rotateText",
                                displayName: "Rotate Text",
                                selector: null,
                                properties: {
                                    show: this.settings.isRotateText,
                                    minAngle: this.settings.minAngle,
                                    maxAngle: this.settings.maxAngle,
                                    maxNumberOfOrientations: this.settings.maxNumberOfOrientations
                                }
                            };
                            instances.push(rotateText);
                            break;
                        }
                        case "stopWords": {
                            var stopWords = {
                                objectName: "stopWords",
                                displayName: "Stop Words",
                                selector: null,
                                properties: {
                                    show: this.settings.isRemoveStopWords,
                                    isDefaultStopWords: this.settings.isDefaultStopWords,
                                    words: this.settings.stopWords ||
                                        this.settings.stopWordsArray.join(WordCloud.StopWordsDelemiter)
                                }
                            };
                            instances.push(stopWords);
                            break;
                        }
                    }
                    return instances;
                };
                WordCloud.prototype.animation = function (element, duration, callback) {
                    if (duration === void 0) { duration = 0; }
                    return element
                        .transition()
                        .duration(duration)
                        .each("end", callback);
                };
                WordCloud.prototype.destroy = function () {
                    this.root = null;
                    this.canvas = null;
                };
                WordCloud.ClassName = "wordCloud";
                WordCloud.Properties = {
                    general: {
                        formatString: {
                            objectName: "general",
                            propertyName: "formatString"
                        },
                        maxNumberOfWords: {
                            objectName: "general",
                            propertyName: "maxNumberOfWords"
                        },
                        minFontSize: {
                            objectName: "general",
                            propertyName: "minFontSize"
                        },
                        maxFontSize: {
                            objectName: "general",
                            propertyName: "maxFontSize"
                        },
                        isBrokenText: {
                            objectName: "general",
                            propertyName: "isBrokenText"
                        },
                    },
                    dataPoint: {
                        fill: {
                            objectName: "dataPoint",
                            propertyName: "fill"
                        }
                    },
                    stopWords: {
                        show: {
                            objectName: "stopWords",
                            propertyName: "show"
                        },
                        isDefaultStopWords: {
                            objectName: "stopWords",
                            propertyName: "isDefaultStopWords"
                        },
                        words: {
                            objectName: "stopWords",
                            propertyName: "words"
                        },
                    },
                    rotateText: {
                        show: {
                            objectName: "rotateText",
                            propertyName: "show"
                        },
                        minAngle: {
                            objectName: "rotateText",
                            propertyName: "minAngle"
                        },
                        maxAngle: {
                            objectName: "rotateText",
                            propertyName: "maxAngle"
                        },
                        maxNumberOfOrientations: {
                            objectName: "rotateText",
                            propertyName: "maxNumberOfOrientations"
                        }
                    }
                };
                WordCloud.Words = {
                    "class": "words",
                    selector: ".words"
                };
                WordCloud.Word = {
                    "class": "word",
                    selector: ".word"
                };
                WordCloud.Size = "px";
                WordCloud.StopWordsDelemiter = " ";
                WordCloud.Radians = Math.PI / 180;
                WordCloud.MinAngle = -180;
                WordCloud.MaxAngle = 180;
                WordCloud.MaxNumberOfWords = 2500;
                WordCloud.MinOpacity = 0.2;
                WordCloud.MaxOpacity = 1;
                WordCloud.capabilities = {
                    dataRoles: [{
                            name: "Category",
                            kind: powerbi.VisualDataRoleKind.Grouping,
                            displayName: "Category"
                        }, {
                            name: "Values",
                            kind: powerbi.VisualDataRoleKind.Measure,
                            displayName: "Values"
                        }],
                    dataViewMappings: [{
                            conditions: [{
                                    "Category": {
                                        min: 1,
                                        max: 1
                                    },
                                    "Values": {
                                        min: 0,
                                        max: 1
                                    }
                                }],
                            categorical: {
                                categories: {
                                    for: { in: "Category" },
                                    dataReductionAlgorithm: { top: { count: WordCloud.MaxNumberOfWords } }
                                },
                                values: {
                                    for: { in: "Values" }
                                }
                            }
                        }],
                    sorting: {
                        implicit: {
                            clauses: [{
                                    role: "Values",
                                    direction: 2
                                }]
                        }
                    },
                    objects: {
                        general: {
                            displayName: "General",
                            properties: {
                                formatString: {
                                    type: {
                                        formatting: {
                                            formatString: true
                                        }
                                    }
                                },
                                maxNumberOfWords: {
                                    displayName: "Max number of words",
                                    type: { numeric: true }
                                },
                                minFontSize: {
                                    displayName: "Min Font",
                                    type: { numeric: true }
                                },
                                maxFontSize: {
                                    displayName: "Max Font",
                                    type: { numeric: true }
                                },
                                isBrokenText: {
                                    displayName: "Word-breaking",
                                    type: { bool: true }
                                },
                                isRemoveStopWords: {
                                    displayName: "Stop Words",
                                    type: { bool: true }
                                }
                            }
                        },
                        dataPoint: {
                            displayName: "Data colors",
                            properties: {
                                fill: {
                                    displayName: "Fill",
                                    type: { fill: { solid: { color: true } } }
                                }
                            }
                        },
                        stopWords: {
                            displayName: "Stop Words",
                            properties: {
                                show: {
                                    displayName: "Show",
                                    type: { bool: true }
                                },
                                isDefaultStopWords: {
                                    displayName: "Default Stop Words",
                                    type: { bool: true }
                                },
                                words: {
                                    displayName: "Words",
                                    type: { text: true }
                                }
                            }
                        },
                        rotateText: {
                            displayName: "Rotate Text",
                            properties: {
                                show: {
                                    displayName: "Show",
                                    type: { bool: true }
                                },
                                minAngle: {
                                    displayName: "Min Angle",
                                    type: { numeric: true }
                                },
                                maxAngle: {
                                    displayName: "Max Angle",
                                    type: { numeric: true }
                                },
                                maxNumberOfOrientations: {
                                    displayName: "Max number of orientations",
                                    type: { numeric: true }
                                }
                            }
                        }
                    }
                };
                WordCloud.Punctuation = [
                    "!", ".", ":", "'", ";", ",", "!",
                    "@", "#", "$", "%", "^", "&", "*",
                    "(", ")", "[", "]", "\"", "\\", "/",
                    "-", "_", "+", "="
                ];
                WordCloud.StopWords = [
                    "a", "able", "about", "across", "after", "all", "almost", "also", "am", "among", "an",
                    "and", "any", "are", "as", "at", "be", "because", "been", "but", "by", "can", "cannot",
                    "could", "did", "do", "does", "either", "else", "ever", "every", "for", "from", "get",
                    "got", "had", "has", "have", "he", "her", "hers", "him", "his", "how", "however", "i",
                    "if", "in", "into", "is", "it", "its", "just", "least", "let", "like", "likely", "may",
                    "me", "might", "most", "must", "my", "neither", "no", "nor", "not", "of", "off", "often",
                    "on", "only", "or", "other", "our", "own", "rather", "said", "say", "says", "she", "should",
                    "since", "so", "some", "than", "that", "the", "their", "them", "then", "there", "these",
                    "they", "this", "tis", "to", "too", "twas", "us", "wants", "was", "we", "were", "what",
                    "when", "where", "which", "while", "who", "whom", "why", "will", "with", "would", "yet",
                    "you", "your"
                ];
                WordCloud.DefaultSettings = {
                    minFontSize: 20,
                    maxFontSize: 100,
                    minAngle: -60,
                    maxAngle: 90,
                    maxNumberOfOrientations: 2,
                    isRotateText: false,
                    isBrokenText: true,
                    isRemoveStopWords: false,
                    stopWordsArray: [],
                    stopWords: undefined,
                    isDefaultStopWords: false,
                    maxNumberOfWords: 200
                };
                WordCloud.RenderDelay = 50;
                WordCloud.DefaultMargin = {
                    top: 10,
                    right: 10,
                    bottom: 10,
                    left: 10
                };
                return WordCloud;
            })();
            WordCloud1447959067750.WordCloud = WordCloud;
            var explore;
            (function (explore) {
                var util;
                (function (util) {
                    function hexToRgb(hex) {
                        var shorthandRegex = /^#?([a-f\d])([a-f\d])([a-f\d])$/i;
                        hex = hex.replace(shorthandRegex, function (m, r, g, b) {
                            return r + r + g + g + b + b;
                        });
                        var result = /^#?([a-f\d]{2})([a-f\d]{2})([a-f\d]{2})$/i.exec(hex);
                        return result ? "rgb(" + parseInt(result[1], 16) + "," + parseInt(result[2], 16) + "," + parseInt(result[3], 16) + ")" : null;
                    }
                    util.hexToRgb = hexToRgb;
                    function getRandomColor() {
                        var red = Math.floor(Math.random() * 255), green = Math.floor(Math.random() * 255), blue = Math.floor(Math.random() * 255);
                        return "rgb(" + red + "," + green + "," + blue + ")";
                    }
                    util.getRandomColor = getRandomColor;
                })(util = explore.util || (explore.util = {}));
            })(explore || (explore = {}));
        })(WordCloud1447959067750 = visuals.WordCloud1447959067750 || (visuals.WordCloud1447959067750 = {}));
    })(visuals = powerbi.visuals || (powerbi.visuals = {}));
})(powerbi || (powerbi = {}));

var powerbi;
(function (powerbi) {
    var visuals;
    (function (visuals) {
        var plugins;
        (function (plugins) {
            plugins.WordCloud1447959067750 = {
                name: 'WordCloud1447959067750',
                class: 'WordCloud1447959067750',
                capabilities: powerbi.visuals.WordCloud1447959067750.WordCloud.capabilities,
                custom: true,
                create: function () { return new powerbi.visuals.WordCloud1447959067750.WordCloud(); }
            };
        })(plugins = visuals.plugins || (visuals.plugins = {}));
    })(visuals = powerbi.visuals || (powerbi.visuals = {}));
})(powerbi || (powerbi = {}));
/*
 *  Power BI Visualizations
 *
 *  Copyright (c) Microsoft Corporation
 *  All rights reserved.
 *  MIT License
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the ""Software""), to deal
 *  in the Software without restriction, including without limitation the rights
 *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED *AS IS*, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 *  THE SOFTWARE.
 */

//# sourceURL=WordCloud.js
//# sourceMappingURL=WordCloud1447959067750.js.map